@@ -38,22 +38,22 @@ cc_new_menu_item (const gchar *label,
 }
 
 GtkWidget *
-cc_new_vbox_widget (gboolean homogenous, gint spacing)
+cc_new_vbox_widget (gint spacing)
 {
 	GtkWidget *vbox;
 
-	vbox = gtk_vbox_new (homogenous, spacing);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, spacing);
 	gtk_widget_show (vbox);
 
 	return vbox;
 }
 
 GtkWidget *
-cc_new_hbox_widget (gboolean homogenous, gint spacing)
+cc_new_hbox_widget (gint spacing)
 {
 	GtkWidget *hbox;
 
-	hbox = gtk_hbox_new (homogenous, spacing);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, spacing);
 	gtk_widget_show (hbox);
 
 	return hbox;
