@@ -49,7 +49,7 @@ provider_config_new (void)
 	GdaDataModel *model;
 
 	priv = g_new0 (ProviderConfigPrivate, 1);
-	provider = gtk_vbox_new (FALSE, 0);
+	provider = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_widget_show (provider);
         gtk_container_set_border_width (GTK_CONTAINER (provider), 6);
 	g_object_set_data_full (G_OBJECT (provider), PROVIDER_CONFIG_DATA, priv,
@@ -86,7 +86,7 @@ provider_config_new (void)
 	gtk_widget_show_all (sw);
 
 	/* add tip */
-	box = gtk_hbox_new (FALSE, 6);
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show (box);
         gtk_container_set_border_width (GTK_CONTAINER (box), 6);
 	gtk_box_pack_start (GTK_BOX (provider), box, FALSE, FALSE, 0);
