@@ -162,7 +162,6 @@ struct _GdauiBasicFormPriv
 
 	GtkWidget  *top_container;
 
-	gboolean    headers_sensitive;
 	gboolean    show_actions;
 	gboolean    entries_auto_default;
 
@@ -297,9 +296,14 @@ gdaui_basic_form_class_init (GdauiBasicFormClass *klass)
 					 g_param_spec_pointer ("paramlist",
 							       _("List of parameters to show in the form"), NULL,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
+	/**
+	 * GdauiBasicForm:headers-sensitive:
+	 *
+	 * Deprecated
+	 */
 	g_object_class_install_property (object_class, PROP_HEADERS_SENSITIVE,
 					 g_param_spec_boolean ("headers-sensitive",
-							       _("Entry headers are sensitive"),
+							       "",
 							       NULL, FALSE,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_SHOW_ACTIONS,
@@ -404,7 +408,6 @@ gdaui_basic_form_init (GdauiBasicForm *wid)
 	wid->priv->place_holders = NULL;
 	wid->priv->top_container = NULL;
 
-	wid->priv->headers_sensitive = FALSE;
 	wid->priv->show_actions = FALSE;
 	wid->priv->entries_auto_default = FALSE;
 
@@ -690,7 +693,6 @@ gdaui_basic_form_set_property (GObject *object,
 			}
 			break;
 		case PROP_HEADERS_SENSITIVE:
-			form->priv->headers_sensitive = g_value_get_boolean (value);
 			break;
 		case PROP_SHOW_ACTIONS:
 			gdaui_basic_form_show_entry_actions (form, g_value_get_boolean (value));
@@ -720,7 +722,6 @@ gdaui_basic_form_get_property (GObject *object,
 			g_value_set_pointer (value, form->priv->set);
 			break;
 		case PROP_HEADERS_SENSITIVE:
-			g_value_set_boolean (value, form->priv->headers_sensitive);
 			break;
 		case PROP_SHOW_ACTIONS:
 			g_value_set_boolean (value, form->priv->show_actions);
@@ -1904,23 +1905,23 @@ real_gdaui_basic_form_entry_set_visible (GdauiBasicForm *form, SingleEntry *sent
 /**
  * gdaui_basic_form_entry_set_visible:
  * @form: a #GdauiBasicForm widget
- * @param: a #GdaHolder object
+ * @holder: a #GdaHolder object
  * @show: set to %TRUE to show the data entry, and to %FALSE to hide it
  *
  * Shows or hides the #GdauiDataEntry in @form which corresponds to the
- * @param parameter
+ * @holder data holder
  *
  * Since: 4.2
  */
 void
-gdaui_basic_form_entry_set_visible (GdauiBasicForm *form, GdaHolder *param, gboolean show)
+gdaui_basic_form_entry_set_visible (GdauiBasicForm *form, GdaHolder *holder, gboolean show)
 {
 	SingleEntry *sentry;
 
 	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
-	g_return_if_fail (GDA_IS_HOLDER (param));
+	g_return_if_fail (GDA_IS_HOLDER (holder));
 
-	sentry = get_single_entry_for_holder (form, param);
+	sentry = get_single_entry_for_holder (form, holder);
 	if (!sentry) {
 		g_warning (_("Can't find data entry for GdaHolder"));
 		return;
@@ -1939,23 +1940,23 @@ gdaui_basic_form_widget_grab_focus (GtkWidget *widget)
 /**
  * gdaui_basic_form_entry_grab_focus:
  * @form: a #GdauiBasicForm widget
- * @param: (allow-none): a #GdaHolder object, or %NULL
+ * @holder: (allow-none): a #GdaHolder object, or %NULL
  *
- * Makes the data entry corresponding to @param grab the focus for the window it's in. If @param is %NULL,
+ * Makes the data entry corresponding to @holder grab the focus for the window it's in. If @holder is %NULL,
  * then the focus is on the first entry which needs attention.
  *
  * Since: 4.2
  */
 void
-gdaui_basic_form_entry_grab_focus (GdauiBasicForm *form, GdaHolder *param)
+gdaui_basic_form_entry_grab_focus (GdauiBasicForm *form, GdaHolder *holder)
 {
 	GtkWidget *entry = NULL;
 
 	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
 
-	if (param) {
-		g_return_if_fail (GDA_IS_HOLDER (param));
-		entry = gdaui_basic_form_get_entry_widget (form, param);
+	if (holder) {
+		g_return_if_fail (GDA_IS_HOLDER (holder));
+		entry = gdaui_basic_form_get_entry_widget (form, holder);
 	}
 
 	if (!entry && form->priv->set) {
@@ -2105,22 +2106,22 @@ get_single_entry_for_id (GdauiBasicForm *form, const gchar *id)
 /**
  * gdaui_basic_form_get_entry_widget:
  * @form: a #GdauiBasicForm widget
- * @param: a #GdaHolder object
+ * @holder: a #GdaHolder object
  *
- * Get the #GdauiDataEntry in @form which corresponds to the param parameter.
+ * Get the #GdauiDataEntry in @form which corresponds to the @holder place.
  *
  * Returns: (transfer none): the requested widget, or %NULL if not found
  *
  * Since: 4.2
  */
 GtkWidget *
-gdaui_basic_form_get_entry_widget (GdauiBasicForm *form, GdaHolder *param)
+gdaui_basic_form_get_entry_widget (GdauiBasicForm *form, GdaHolder *holder)
 {
 	SingleEntry *sentry;
 	g_return_val_if_fail (GDAUI_IS_BASIC_FORM (form), NULL);
-	g_return_val_if_fail (GDA_IS_HOLDER (param), NULL);
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 
-	sentry = get_single_entry_for_holder (form, param);
+	sentry = get_single_entry_for_holder (form, holder);
 	if (sentry)
 		return GTK_WIDGET (sentry->entry);
 	else
@@ -2130,22 +2131,22 @@ gdaui_basic_form_get_entry_widget (GdauiBasicForm *form, GdaHolder *param)
 /**
  * gdaui_basic_form_get_label_widget:
  * @form: a #GdauiBasicForm widget
- * @param: a #GdaHolder object
+ * @holder: a #GdaHolder object
  *
- * Get the label in @form which corresponds to the param parameter.
+ * Get the label in @form which corresponds to the @holder holder.
  *
  * Returns: (transfer none): the requested widget, or %NULL if not found
  *
  * Since: 4.2
  */
 GtkWidget *
-gdaui_basic_form_get_label_widget (GdauiBasicForm *form, GdaHolder *param)
+gdaui_basic_form_get_label_widget (GdauiBasicForm *form, GdaHolder *holder)
 {
 	SingleEntry *sentry;
 	g_return_val_if_fail (GDAUI_IS_BASIC_FORM (form), NULL);
-	g_return_val_if_fail (GDA_IS_HOLDER (param), NULL);
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 
-	sentry = get_single_entry_for_holder (form, param);
+	sentry = get_single_entry_for_holder (form, holder);
 	if (sentry)
 		return sentry->label;
 	else
