@@ -114,7 +114,7 @@ gdaui_cloud_get_type (void)
                         NULL
                 };
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiCloud", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiCloud", &info, 0);
 		g_type_add_interface_static (type, GDAUI_TYPE_DATA_SELECTOR, &selector_info);
 	}
 
@@ -125,11 +125,11 @@ static void
 cloud_map (GtkWidget *widget)
 {
         GTK_WIDGET_CLASS (parent_class)->map (widget);
-	GtkStyle *style;
-	GdkColor color;
-	style = gtk_widget_get_style (widget);
-	color = style->bg[GTK_STATE_NORMAL];
-	gtk_widget_modify_base (GDAUI_CLOUD (widget)->priv->tview, GTK_STATE_NORMAL, &color);
+	GtkStyleContext *style_context;
+	GdkRGBA color;
+	style_context = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_background_color (style_context, GTK_STATE_FLAG_NORMAL, &color);
+	gtk_widget_override_background_color (GDAUI_CLOUD (widget)->priv->tview, GTK_STATE_FLAG_NORMAL, &color);
 }
 
 static void
@@ -347,6 +347,8 @@ gdaui_cloud_init (GdauiCloud *cloud)
 	cloud->priv->selected_tags = NULL;
 	cloud->priv->selection_mode = GTK_SELECTION_SINGLE;
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (cloud), GTK_ORIENTATION_VERTICAL);
+
 	/* text buffer */
         cloud->priv->tbuffer = gtk_text_buffer_new (NULL);
         gtk_text_buffer_create_tag (cloud->priv->tbuffer, "section",
@@ -360,7 +362,7 @@ gdaui_cloud_init (GdauiCloud *cloud)
                                         GTK_POLICY_AUTOMATIC);
         gtk_box_pack_start (GTK_BOX (cloud), sw, TRUE, TRUE, 0);
 
-        vbox = gtk_vbox_new (FALSE, 0);
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	vp = gtk_viewport_new (NULL, NULL);
         gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);
 
@@ -1062,7 +1064,7 @@ gdaui_cloud_create_filter_widget (GdauiCloud *cloud)
 	GtkWidget *hbox, *label, *wid;
 	g_return_val_if_fail (GDAUI_IS_CLOUD (cloud), NULL);
 	
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	label = gtk_label_new (_("Find:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -1079,8 +1081,8 @@ gdaui_cloud_create_filter_widget (GdauiCloud *cloud)
 /**
  * gdaui_cloud_set_weight_func:
  * @cloud: a #GdauiCloud widget
- * @func: (allow-none): a #GdauiCloudWeightFunc function which computes weights, or %NULL to unset
- * @data: (allow-none): a pointer to pass as last argument of @func each time it is called
+ * @func: (allow-none) (scope notified): a #GdauiCloudWeightFunc function which computes weights, or %NULL to unset
+ * @data: (allow-none): a pointer to pass as last argument of @func each time it is called, or %NULL
  *
  * Specifies a function called by @cloud to compute each row's respective weight.
  *
