@@ -97,7 +97,7 @@ gdaui_init (void)
  * if @plugin_name is not %NULL (if no entry of that name is found, then the default data
  * entry widget will be created).
  *
- * @plugin_name format is interpreted as two parts: &lt;plugin name&gt;:&lt;plugin options&gt;, and
+ * The @plugin_name format is interpreted as two parts: &lt;plugin name&gt;:&lt;plugin options&gt;, and
  * if the plugins has no option, then the ":&lt;plugin options&gt;" part may be omitted.
  * 
  * Returns: (transfer full): a new #GdauiDataEntry widget, _NEVER_ %NULL
@@ -112,7 +112,7 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 	if (!gdaui_plugins_hash)
 		gdaui_plugins_hash = init_plugins_hash ();
 
-	if (type == G_TYPE_INVALID)
+	if (type == GDA_TYPE_NULL)
 		return (GdauiDataEntry *) gdaui_entry_none_new (GDA_TYPE_NULL);
 
 	dh = gda_data_handler_get_default (type);
@@ -171,8 +171,7 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 			 (type == GDA_TYPE_BINARY))
 			entry = (GdauiDataEntry *) gdaui_entry_bin_new (dh, type);
 		else if	((type == GDA_TYPE_GEOMETRIC_POINT) ||
-			 (type == G_TYPE_OBJECT) ||
-			 (type == GDA_TYPE_LIST))
+			 (type == G_TYPE_OBJECT))
 			entry = (GdauiDataEntry *) gdaui_entry_none_new (type);
 		else if	(type == GDA_TYPE_TIME)
 			entry = (GdauiDataEntry *) gdaui_entry_time_new (dh);
@@ -716,7 +715,7 @@ get_spec_with_isocodes (const gchar *file)
 		xmlFreeNode (node);
 
 		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameters", NULL, NULL);
-		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameter", "id", "CURRENCY");
+		node = find_child_node_from_name (node, "parameter", "id", "CURRENCY");
 		xmlSetProp (node, BAD_CAST "source", NULL);
 	}
 
