@@ -88,6 +88,7 @@ struct _GdauiRtEditorPriv
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
+static GtkCssProvider *css_provider = NULL;
 static gchar *help_str=N_("\"\"\"= Title level 1 =\n"
 			  "== Title level 2 ==\n"
 			  "\"\"\"= Title level 1 =\n"
@@ -199,7 +200,7 @@ gdaui_rt_editor_get_type (void)
 			0
 		};
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiRtEditor", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiRtEditor", &info, 0);
 	}
 
 	return type;
@@ -300,6 +301,8 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 {
 	GtkWidget *sw, *textview, *toolbar;
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (rte), GTK_ORIENTATION_VERTICAL);
+
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -399,6 +402,18 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 
 	/* toolbar */
 	toolbar = gtk_ui_manager_get_widget (rte->priv->uimanager, "/ToolBar");
+
+	if (!css_provider) {
+		css_provider = gtk_css_provider_new ();
+		gtk_css_provider_load_from_data (css_provider,
+						 "* {\n"
+						 "-GtkToolbar-shadow-type : none}",
+						 -1, NULL);
+	}
+	gtk_style_context_add_provider (gtk_widget_get_style_context ((GtkWidget*) toolbar),
+					GTK_STYLE_PROVIDER (css_provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
 	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
 	rte->priv->toolbar = toolbar;
 	gtk_box_pack_end (GTK_BOX (rte), toolbar, FALSE, FALSE, 0);
@@ -855,7 +870,7 @@ bigger_font_item_activate_cb (G_GNUC_UNUSED GtkCheckMenuItem *checkmenuitem,
 	fd = pango_context_get_font_description (pcontext);
 	nfd = pango_font_description_copy_static (fd);
 	pango_font_description_set_size (nfd, pango_font_description_get_size (fd) * 1.2);
-	gtk_widget_modify_font (GTK_WIDGET (rte->priv->textview), nfd);
+	gtk_widget_override_font (GTK_WIDGET (rte->priv->textview), nfd);
 	pango_font_description_free (nfd);
 }
 
@@ -869,7 +884,7 @@ smaller_font_item_activate_cb (G_GNUC_UNUSED GtkCheckMenuItem *checkmenuitem,
 	fd = pango_context_get_font_description (pcontext);
 	nfd = pango_font_description_copy_static (fd);
 	pango_font_description_set_size (nfd, pango_font_description_get_size (fd) / 1.2);
-	gtk_widget_modify_font (GTK_WIDGET (rte->priv->textview), nfd);
+	gtk_widget_override_font (GTK_WIDGET (rte->priv->textview), nfd);
 	pango_font_description_free (nfd);
 }
 
@@ -877,7 +892,7 @@ static void
 reset_font_item_activate_cb (G_GNUC_UNUSED GtkCheckMenuItem *checkmenuitem,
 			     GdauiRtEditor *rte)
 {
-	gtk_widget_modify_font (GTK_WIDGET (rte->priv->textview), NULL);
+	gtk_widget_override_font (GTK_WIDGET (rte->priv->textview), NULL);
 }
 
 static void
@@ -1065,14 +1080,7 @@ apply_markup (GdauiRtEditor *rte, GtkTextBuffer *textbuffer, TextTag *current, G
 		pe = start;
 		data = remove_newlines_from_base64 (gtk_text_buffer_get_text (textbuffer, &ps, &pe, FALSE));
 		/*g_print ("{{{%s}}}\n", data);*/
-#if GLIB_CHECK_VERSION(2,20,0)
 		g_base64_decode_inplace (data, &length);
-#else
-		gchar *tmp;
-		tmp = g_base64_decode (data, &length);
-		g_free (data);
-		data = tmp;
-#endif
 		if (gdk_pixdata_deserialize (&pixdata, length, (guint8*) data, NULL)) {
 			GdkPixbuf *pixbuf;
 			pixbuf = gdk_pixbuf_from_pixdata (&pixdata, TRUE, NULL);
@@ -1208,6 +1216,8 @@ text_buffer_changed_cb (GtkTextBuffer *textbuffer, GdauiRtEditor *rte)
  *
  * returns the token type starting from @iter, and positions @out_end to the last used position
  * position.
+ *
+ * Returns: a #MarkupTag
  */
 static MarkupTag
 get_token (GtkTextIter *iter, gint *out_nb_spaces_before, GtkTextIter *out_end,
