@@ -1,26 +1,31 @@
 /*
- * Copyright (C) 1998 - 2011 The GNOME Foundation.
+ * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
+ * Copyright (C) 2000 - 2004 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2002 Cleber Rodrigues <cleberrrjr@bol.com.br>
+ * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@ximian.com>
+ * Copyright (C) 2003 Filip Van Raemdonck <mechanix@debian.org>
+ * Copyright (C) 2004 - 2005 Alan Knowles <alank@src.gnome.org>
+ * Copyright (C) 2004 Jos Mara Casanova Crespo <jmcasanova@igalia.com>
+ * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Michael Lausch <michael@lausch.at>
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *	Bas Driessen <bas.driessen@xobas.com>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GDA_CONNECTION_H__
@@ -57,7 +62,8 @@ typedef enum {
 	GDA_CONNECTION_CANT_LOCK_ERROR,
 	GDA_CONNECTION_TASK_NOT_FOUND_ERROR,
 	GDA_CONNECTION_UNSUPPORTED_THREADS_ERROR,
-	GDA_CONNECTION_CLOSED_ERROR
+	GDA_CONNECTION_CLOSED_ERROR,
+	GDA_CONNECTION_META_DATA_CONTEXT_ERROR
 } GdaConnectionError;
 
 #define GDA_CONNECTION_NONEXIST_DSN_ERROR GDA_CONNECTION_DSN_NOT_FOUND_ERROR
@@ -194,7 +200,7 @@ typedef enum {
 
 
 /**
- * GdaConnectionFeature
+ * GdaConnectionFeature:
  * @GDA_CONNECTION_FEATURE_AGGREGATES: test for aggregates support
  * @GDA_CONNECTION_FEATURE_BLOBS: test for BLOBS (binary large objects) support
  * @GDA_CONNECTION_FEATURE_INDEXES: test for indexes support
@@ -268,6 +274,11 @@ GdaConnection       *gda_connection_open_from_dsn        (const gchar *dsn, cons
 GdaConnection       *gda_connection_open_from_string     (const gchar *provider_name, 
 							  const gchar *cnc_string, const gchar *auth_string,
 							  GdaConnectionOptions options, GError **error);
+GdaConnection       *gda_connection_new_from_dsn         (const gchar *dsn, const gchar *auth_string,
+							  GdaConnectionOptions options, GError **error);
+GdaConnection       *gda_connection_new_from_string      (const gchar *provider_name, 
+							  const gchar *cnc_string, const gchar *auth_string,
+							  GdaConnectionOptions options, GError **error);
 gboolean             gda_connection_open                 (GdaConnection *cnc, GError **error);
 void                 gda_connection_close                (GdaConnection *cnc);
 void                 gda_connection_close_no_warning     (GdaConnection *cnc);
