@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -110,7 +110,7 @@ static const GValue        *gda_data_meta_wrapper_get_value_at    (GdaDataModel
 static GObjectClass *parent_class = NULL;
 
 /**
- * gda_data_meta_wrapper_get_type
+ * gda_data_meta_wrapper_get_type:
  *
  * Returns: the #GType of GdaDataMetaWrapper.
  */
@@ -328,7 +328,7 @@ gda_data_meta_wrapper_get_property (GObject *object,
 }
 
 /**
- * gda_data_meta_wrapper_new
+ * gda_data_meta_wrapper_new:
  * @model: a #GdaDataModel
  *
  * Creates a new #GdaDataModel object which buffers the rows of @model. This object is useful
@@ -468,7 +468,7 @@ to_lower (gchar *str)
 }
 
 /**
- * _gda_data_meta_wrapper_compute_value
+ * _gda_data_meta_wrapper_compute_value:
  *
  * Returns:
  *  - NULL if no changes are necessary from the current value
@@ -493,7 +493,7 @@ _gda_data_meta_wrapper_compute_value (const GValue *value, GdaSqlIdentifierStyle
 		gboolean onechanged = FALSE;
 		for (i = 0; sa[i]; i++) {
 			if (identifier_needs_quotes (sa[i], mode)) {
-				gchar *tmp = gda_sql_identifier_add_quotes (sa[i]);
+				gchar *tmp = gda_sql_identifier_force_quotes (sa[i]);
 				g_free (sa[i]);
 				sa[i] = tmp;
 				onechanged = TRUE;
@@ -506,7 +506,7 @@ _gda_data_meta_wrapper_compute_value (const GValue *value, GdaSqlIdentifierStyle
 
 				if ((reserved_keyword_func && reserved_keyword_func (sa[i])) ||
 				    (! reserved_keyword_func && is_keyword (sa[i]))) {
-					gchar *tmp = gda_sql_identifier_add_quotes (sa[i]);
+					gchar *tmp = gda_sql_identifier_force_quotes (sa[i]);
 					g_free (sa[i]);
 					sa[i] = tmp;
 					onechanged = TRUE;
@@ -521,7 +521,7 @@ _gda_data_meta_wrapper_compute_value (const GValue *value, GdaSqlIdentifierStyle
 	else {
 		if (identifier_needs_quotes (str, mode)) {
 			retval = gda_value_new (G_TYPE_STRING);
-			g_value_take_string (retval, gda_sql_identifier_add_quotes (str));
+			g_value_take_string (retval, gda_sql_identifier_force_quotes (str));
 		}
 		else {
 			gchar *tmp = NULL;
@@ -530,7 +530,7 @@ _gda_data_meta_wrapper_compute_value (const GValue *value, GdaSqlIdentifierStyle
 
 			if ((reserved_keyword_func && reserved_keyword_func (tmp ? tmp : str)) ||
 			    (! reserved_keyword_func && is_keyword (tmp ? tmp : str))) {
-				gchar *tmp2 = gda_sql_identifier_add_quotes (tmp ? tmp : str);
+				gchar *tmp2 = gda_sql_identifier_force_quotes (tmp ? tmp : str);
 				if (tmp)
 					g_free (tmp);
 				tmp = tmp2;
