@@ -251,7 +251,8 @@ add_error (GdaDataModelDir *model, const gchar *err)
 {
 	GError *error = NULL;
 
-        g_set_error (&error, 0, 0, "%s", err);
+        g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+		     "%s", err);
         model->priv->errors = g_slist_append (model->priv->errors, error);
 }
 
@@ -687,7 +688,7 @@ gda_data_model_dir_new (const gchar *basedir)
  *
  * Get the list of errors which have occurred while using @model
  *
- * Returns: (transfer none) (element-type GObject.Error) : a read-only list of #GError pointers, or %NULL if no error has occurred
+ * Returns: (transfer none) (element-type GLib.Error) : a read-only list of #GError pointers, or %NULL if no error has occurred
  */
 const GSList *
 gda_data_model_dir_get_errors (GdaDataModelDir *model)
@@ -925,7 +926,8 @@ gda_data_model_dir_set_value_at (GdaDataModel *model, gint col, gint row, const
 		gchar *tmp;
 		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
 		add_error (imodel, tmp);
-		g_set_error (error, 0, 0, "%s", tmp);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     "%s", tmp);
 		g_free (tmp);
 		return FALSE;
 	}
@@ -971,7 +973,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 		else
 			str = g_strdup_printf (_("Row %d not found (empty data model)"), row);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, "%s", str);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+			     "%s", str);
 		g_free (str);
                 return FALSE;
         }
@@ -992,8 +995,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 		case COL_MD5SUM:
 		default:
 			add_error (imodel, _("Column cannot be modified"));
-			g_set_error (error, 0, 0, "%s",
-				     _("Column cannot be modified"));
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+				     "%s", _("Column cannot be modified"));
 			return FALSE;
 		case COL_DIRNAME: {
 			/* check that the new dir still starts with the basedir */
@@ -1006,8 +1009,10 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 			base_len = strlen (imodel->priv->basedir);
 			if ((len < base_len) ||
 			    (strncmp (new_path, imodel->priv->basedir, base_len))) {
-				add_error (imodel, _("New path must be a subpath of the base directory"));
-				g_set_error (error, 0, 0, "%s",
+				add_error (imodel,
+					   _("New path must be a subpath of the base directory"));
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     "%s",
 					     _("New path must be a subpath of the base directory"));
 				return FALSE;
 			}
@@ -1051,7 +1056,9 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 					str = g_strdup_printf (_("Could not rename file '%s' to '%s'"),
 							       filename, new_filename);
 					add_error (imodel, str);
-					g_set_error (error, 0, 0, "%s", str);
+					g_set_error (error,
+						     GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+						     "%s", str);
 					g_free (str);
 					g_free (new_filename);
 					g_free (filename);
@@ -1074,7 +1081,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 				gchar *str;
 				str = g_strdup_printf (_("Could not create directory '%s'"), new_path);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, "%s", str);
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     "%s", str);
 				g_free (str);
 				g_free (old_path);
 				return FALSE;
@@ -1094,7 +1102,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 				gchar *str;
 				str = g_strdup_printf (_("Could not rename file '%s' to '%s'"), filename, new_filename);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, "%s", str);
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     "%s", str);
 				g_free (str);
 				g_free (new_filename);
 				g_free (filename);
@@ -1143,7 +1152,9 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 					gchar *str;
 					str = g_strdup_printf (_("Could not overwrite contents of file '%s'"), filename);
 					add_error (imodel, str);
-					g_set_error (error, 0, 0, "%s", str);
+					g_set_error (error, GDA_DATA_MODEL_ERROR,
+						     GDA_DATA_MODEL_ACCESS_ERROR,
+						     "%s", str);
 					g_free (str);
 					g_object_unref (op);
 					g_free (filename);
@@ -1160,7 +1171,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 			}
 			else {
 				add_error (imodel, _("Wrong type of data"));
-				g_set_error (error, 0, 0, "%s", _("Wrong type of data"));
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     "%s", _("Wrong type of data"));
 				return FALSE;
 			}
 			break;
@@ -1202,7 +1214,8 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 		case COL_MD5SUM:
 		default:
 			add_error (imodel, _("Column cannot be set"));
-			g_set_error (error, 0, 0, "%s",
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+				     "%s",
 				     _("Column cannot be set"));
 			return -1;
 		case COL_DIRNAME:
@@ -1214,7 +1227,9 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 				if ((len < base_len) ||
 				    (strncmp (dirname, imodel->priv->basedir, base_len))) {
 					add_error (imodel, _("New path must be a subpath of the base directory"));
-					g_set_error (error, 0, 0, "%s",
+					g_set_error (error, GDA_DATA_MODEL_ERROR,
+						     GDA_DATA_MODEL_ACCESS_ERROR,
+						     "%s",
 						     _("New path must be a subpath of the base directory"));
 					return -1;
 				}
@@ -1276,7 +1291,8 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 				gchar *str;
 				str = g_strdup_printf (_("Cannot set contents of filename '%s'"), complete_filename);
 				add_error (imodel, str);
-				g_set_error (error, 0, 0, "%s", str);
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     "%s", str);
 				g_free (str);
 				if (bin_to_free)
 					g_free (bin_data);
@@ -1287,15 +1303,16 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 			gchar *str;
 			str = g_strdup_printf (_("Cannot create directory '%s'"), dirname);
 			add_error (imodel, str);
-			g_set_error (error, 0, 0, "%s", str);
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+				     "%s", str);
 			g_free (str);
 			return -1;
 		}
 	}
 	else {
 		add_error (imodel, _("Cannot add row: filename missing"));
-		g_set_error (error, 0, 0, "%s",
-			     _("Cannot add row: filename missing"));
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     "%s", _("Cannot add row: filename missing"));
 		return -1;
 	}
 
@@ -1322,7 +1339,8 @@ gda_data_model_dir_remove_row (GdaDataModel *model, gint row, GError **error)
 		else
 			str = g_strdup_printf (_("Row %d not found (empty data model)"), row);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, "%s", str);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     "%s", str);
 		g_free (str);
                 return FALSE;
         }
@@ -1338,7 +1356,8 @@ gda_data_model_dir_remove_row (GdaDataModel *model, gint row, GError **error)
 		gchar *str;
 		str = g_strdup_printf (_("Cannot remove file '%s'"), filename);
 		add_error (imodel, str);
-		g_set_error (error, 0, 0, "%s", str);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     "%s", str);
 		g_free (str);
 		g_free (filename);
 		return FALSE;
