@@ -140,7 +140,7 @@ gda_data_model_iter_class_init (GdaDataModelIterClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	/**
-	 * GdaDataModelIter::row-changed
+	 * GdaDataModelIter::row-changed:
 	 * @iter: the #GdaDataModelIter
 	 * @row: the new iter's row
 	 *
@@ -154,7 +154,7 @@ gda_data_model_iter_class_init (GdaDataModelIterClass *class)
                               NULL, NULL,
                               g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 	/**
-	 * GdaDataModelIter::end-of-data
+	 * GdaDataModelIter::end-of-data:
 	 * @iter: the #GdaDataModelIter
 	 *
 	 * Gets emitted when @iter has reached the end of available data (which means the previous
@@ -247,9 +247,9 @@ model_row_removed_cb (G_GNUC_UNUSED GdaDataModel *model, gint row, GdaDataModelI
 static void
 model_reset_cb (GdaDataModel *model, GdaDataModelIter *iter)
 {
-	/* reset the iter to before the 1st row */
-	gda_data_model_iter_invalidate_contents (iter);
-	gda_data_model_iter_move_to_row (iter, -1);
+	gint row;
+
+	row = gda_data_model_iter_get_row (iter);
 	
 	/* adjust GdaHolder's type if a column's type has changed from GDA_TYPE_NULL
 	 * to something else */
@@ -266,6 +266,13 @@ model_reset_cb (GdaDataModel *model, GdaDataModelIter *iter)
 					      gda_column_get_g_type (col), NULL);
 		}
 	}
+
+	gda_data_model_iter_invalidate_contents (iter);
+	if (row >= 0)
+		gda_data_model_iter_move_to_row (iter, row);
+	else
+		/* reset the iter to before the 1st row */
+		gda_data_model_iter_move_to_row (iter, -1);
 }
 
 /*
@@ -577,14 +584,18 @@ gda_data_model_iter_get_property (GObject *object,
  *
  * Synchronizes the values of the parameters in @iter with the values at the @row row.
  *
- * If @row is not a valid row, then the returned value is FALSE, and the "current-row"
- * property is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
+ * If @row is not a valid row, then the returned value is %FALSE, and the "current-row"
+ * property is set to -1 (which means that gda_data_model_iter_is_valid() would return %FALSE),
+ * with the exception that if @row is -1, then the returned value is %TRUE.
  *
- * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the @row row.  In this case
- * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
+ * This function can return %FALSE if it was not allowed to be moved (as it emits the
+ * "validate-set" signal before being moved).
  *
- * Returns: TRUE if no error occurred
+ * When this function returns %TRUE, then @iter has actually been moved to the next row,
+ * but some values may not have been read correctly in the row, in which case the
+ * correcsponding #GdaHolder will be left invalid.
+ *
+ * Returns: %TRUE if no error occurred
  */
 gboolean
 gda_data_model_iter_move_to_row (GdaDataModelIter *iter, gint row)
@@ -592,8 +603,25 @@ gda_data_model_iter_move_to_row (GdaDataModelIter *iter, gint row)
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
 	g_return_val_if_fail (iter->priv, FALSE);
 
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    (gda_data_model_iter_get_row (iter) == row)) {
+		/* refresh @iter's contents */
+		GdaDataModel *model;
+		model = iter->priv->data_model;
+		g_return_val_if_fail (model, FALSE);
+
+		if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
+			return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter,
+										  row);
+		else
+			return gda_data_model_iter_move_to_row_default (model, iter, row);
+	}
+
 	if (row < 0) {
-		if (iter->priv->row != -1) {
+		if (gda_data_model_iter_get_row (iter) >= 0) {
+			if (! _gda_set_validate ((GdaSet*) iter, NULL))
+				return FALSE;
+
 			iter->priv->row = -1;
 			g_signal_emit (G_OBJECT (iter),
 				       gda_data_model_iter_signals[ROW_CHANGED],
@@ -603,44 +631,18 @@ gda_data_model_iter_move_to_row (GdaDataModelIter *iter, gint row)
 	}
 	else {
 		GdaDataModel *model;
-		if ((gda_data_model_iter_get_row (iter) >= 0) &&
-		    (gda_data_model_iter_get_row (iter) != row) && 
-		    ! _gda_set_validate ((GdaSet*) iter, NULL)) {
-			return FALSE;
-		}
-
-		gboolean *null_oks = NULL;
-		if (GDA_SET (iter)->holders) {
-			gint i;
-			GSList *list;
-
-			null_oks = g_new (gboolean, g_slist_length (GDA_SET (iter)->holders));
-			for (i = 0, list = GDA_SET (iter)->holders; list; i++, list = list->next) {
-				null_oks[i] = gda_holder_get_not_null ((GdaHolder*) list->data);
-				gda_holder_set_not_null ((GdaHolder*) list->data, FALSE);
-			}
-		}
-
-		gboolean move_ok;
 		model = iter->priv->data_model;
-		g_return_val_if_fail (iter->priv->data_model, FALSE);
+		g_return_val_if_fail (model, FALSE);
 
-		if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
-			move_ok = (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter, row);
-		else {
-			/* default method */
-			move_ok = gda_data_model_iter_move_to_row_default (model, iter, row);
+		if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) {
+			if ((gda_data_model_iter_get_row (iter) >= 0) &&
+			    ! _gda_set_validate ((GdaSet*) iter, NULL))
+				return FALSE;
+			return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter,
+										  row);
 		}
-		
-		if (null_oks) {
-			gint i;
-			GSList *list;
-			for (i = 0, list = GDA_SET (iter)->holders; list; i++, list = list->next)
-				gda_holder_set_not_null ((GdaHolder*) list->data, null_oks[i]);
-			g_free (null_oks);
-		}
-		
-		return move_ok;
+		else
+			return gda_data_model_iter_move_to_row_default (model, iter, row);
 	}
 }
 
@@ -658,6 +660,13 @@ set_param_attributes (GdaHolder *holder, GdaValueAttribute flags)
 
 /**
  * gda_data_model_iter_move_to_row_default: (skip)
+ * @model: a #GdaDataModel
+ * @iter: a #GdaDataModelIter iterating in @model
+ * @row: the requested row
+ * 
+ * Method reserved to #GdaDataModelIter implementations, should not be called directly.
+ *
+ * Returns: %TRUE if @iter was moved correctly.
  */
 gboolean
 gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row)
@@ -667,9 +676,10 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
 	gint col;
 	GdaDataModel *test;
 	gboolean update_model;
-	gboolean retval = TRUE;
 	
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    ! _gda_set_validate ((GdaSet*) iter, NULL))
+		return FALSE;
 
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
 		return FALSE;
@@ -692,18 +702,17 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
 		const GValue *cvalue;
-		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		GError *lerror = NULL;
+		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
 		if (!cvalue || 
-		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
-			gda_holder_force_invalid ((GdaHolder*) list->data);
-			retval = FALSE;
-		}
+		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, &lerror))
+			gda_holder_force_invalid_e ((GdaHolder*) list->data, lerror);
 		else
 			set_param_attributes ((GdaHolder*) list->data, 
 					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, "update-model", update_model, NULL);
-	return retval;
+	return TRUE;
 }
 
 
@@ -715,14 +724,17 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
  * (synchronizes the values of the parameters in @iter with the values at the new row).
  *
  * If the iterator was on the data model's last row, then it can't be moved forward
- * anymore, and the returned value is FALSE; nore also that the "current-row" property
- * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
+ * anymore, and the returned value is %FALSE; note also that the "current-row" property
+ * is set to -1 (which means that gda_data_model_iter_is_valid() would return %FALSE)
  *
- * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the new current row (one row more than it was before the call). In this case
- * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
+ * This function can return %FALSE if it was not allowed to be moved (as it emits the
+ * "validate-set" signal before being moved).
  *
- * Returns: TRUE if the iterator is now at the next row
+ * When this function returns %TRUE, then @iter has actually been moved to the next row,
+ * but some values may not have been read correctly in the row, in which case the
+ * correcsponding #GdaHolder will be left invalid.
+ *
+ * Returns: %TRUE if the iterator is now at the next row
  */
 gboolean
 gda_data_model_iter_move_next (GdaDataModelIter *iter)
@@ -732,13 +744,13 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
 	g_return_val_if_fail (iter->priv, FALSE);
 	g_return_val_if_fail (iter->priv->data_model, FALSE);
 
-	if ((gda_data_model_iter_get_row (iter) >= 0) &&
-	    ! _gda_set_validate ((GdaSet*) iter, NULL))
-		return FALSE;
-
 	model = iter->priv->data_model;
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next)
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next) {
+		if ((gda_data_model_iter_get_row (iter) >= 0) &&
+		    ! _gda_set_validate ((GdaSet*) iter, NULL))
+			return FALSE;
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next) (model, iter);
+	}
 	else 
 		/* default method */
 		return gda_data_model_iter_move_next_default (model, iter);
@@ -746,6 +758,12 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
 
 /**
  * gda_data_model_iter_move_next_default: (skip)
+ * @model: a #GdaDataModel
+ * @iter: a #GdaDataModelIter iterating in @model
+ * 
+ * Method reserved to #GdaDataModelIter implementations, should not be called directly.
+ *
+ * Returns: %TRUE if @iter was moved correctly.
  */
 gboolean
 gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *iter)
@@ -755,9 +773,11 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
-	gboolean retval = TRUE;
-	
-	/* validity tests */
+
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    ! _gda_set_validate ((GdaSet*) iter, NULL))
+		return FALSE;
+
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
 		return FALSE;
 	
@@ -780,19 +800,18 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
 		const GValue *cvalue;
-		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		GError *lerror = NULL;
+		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
 		if (!cvalue || 
-		    !gda_holder_set_value ((GdaHolder *) list->data, cvalue, NULL)) {
-			gda_holder_force_invalid ((GdaHolder *) list->data);
-			retval = FALSE;
-		}
+		    !gda_holder_set_value ((GdaHolder *) list->data, cvalue, &lerror))
+			gda_holder_force_invalid_e ((GdaHolder *) list->data, lerror);
 		else
 			set_param_attributes ((GdaHolder *) list->data, 
 					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, 
 		      "update-model", update_model, NULL);
-	return retval;
+	return TRUE;
 }
 
 /**
@@ -803,14 +822,17 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
  * with the values at the new row).
  *
  * If the iterator was on the data model's first row, then it can't be moved backwards
- * anymore, and the returned value is FALSE; note also that the "current-row" property
- * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE).
+ * anymore, and the returned value is %FALSE; note also that the "current-row" property
+ * is set to -1 (which means that gda_data_model_iter_is_valid() would return %FALSE).
  *
- * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the new current row (one row less than it was before the call).  In this case
- * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
+ * This function can return %FALSE if it was not allowed to be moved (as it emits the
+ * "validate-set" signal before being moved).
  *
- * Returns: TRUE if the iterator is now at the previous row
+ * When this function returns %TRUE, then @iter has actually been moved to the next row,
+ * but some values may not have been read correctly in the row, in which case the
+ * correcsponding #GdaHolder will be left invalid.
+ *
+ * Returns: %TRUE if the iterator is now at the previous row
  */
 gboolean
 gda_data_model_iter_move_prev (GdaDataModelIter *iter)
@@ -821,13 +843,13 @@ gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 	g_return_val_if_fail (iter->priv, FALSE);
 	g_return_val_if_fail (iter->priv->data_model, FALSE);
 
-	if ((gda_data_model_iter_get_row (iter) >= 0) &&
-	    ! _gda_set_validate ((GdaSet*) iter, NULL))
-		return FALSE;
-
 	model = iter->priv->data_model;
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev)
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev) {
+		if ((gda_data_model_iter_get_row (iter) >= 0) &&
+		    ! _gda_set_validate ((GdaSet*) iter, NULL))
+			return FALSE;
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev) (model, iter);
+	}
 	else 
 		/* default method */
 		return gda_data_model_iter_move_prev_default (model, iter);
@@ -835,6 +857,12 @@ gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 
 /**
  * gda_data_model_iter_move_prev_default: (skip)
+ * @model: a #GdaDataModel
+ * @iter: a #GdaDataModelIter iterating in @model
+ * 
+ * Method reserved to #GdaDataModelIter implementations, should not be called directly.
+ *
+ * Returns: %TRUE if @iter was moved correctly.
  */
 gboolean
 gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *iter)
@@ -844,9 +872,11 @@ gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
-	gboolean retval = TRUE;
 	
-	/* validity tests */
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    ! _gda_set_validate ((GdaSet*) iter, NULL))
+		return FALSE;
+
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
 		return FALSE;
 	
@@ -869,19 +899,18 @@ gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
 	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
 		const GValue *cvalue;
-		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		GError *lerror = NULL;
+		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
 		if (!cvalue || 
-		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
-			gda_holder_force_invalid ((GdaHolder*) list->data);
-			retval = FALSE;
-		}
+		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, &lerror))
+			gda_holder_force_invalid_e ((GdaHolder*) list->data, lerror);
 		else
 			set_param_attributes ((GdaHolder*) list->data,
 					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, 
 		      "update-model", update_model, NULL);
-	return retval;
+	return TRUE;
 }
 
 
@@ -1005,6 +1034,36 @@ gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
 		return NULL;
 }
 
+/**
+ * gda_data_model_iter_get_value_at_e:
+ * @iter: a #GdaDataModelIter object
+ * @col: the requested column
+ * @error: (allow-none): a place to store errors, or %NULL
+ *
+ * Get the value stored at the column @col in @iter. The returned value must not be modified.
+ *
+ * Returns: (transfer none): the #GValue, or %NULL if the value could not be fetched
+ *
+ * Since: 4.2.10
+ */
+const GValue *
+gda_data_model_iter_get_value_at_e (GdaDataModelIter *iter, gint col, GError **error)
+{
+	GdaHolder *param;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), NULL);
+	g_return_val_if_fail (iter->priv, NULL);
+
+	param = (GdaHolder *) g_slist_nth_data (((GdaSet *) iter)->holders, col);
+	if (param) {
+		if (error)
+			gda_holder_is_valid_e (param, error);
+		return gda_holder_get_value (param);
+	}
+	else
+		return NULL;
+}
+
 /**
  * gda_data_model_iter_set_value_at:
  * @iter: a #GdaDataModelIter object
