@@ -58,8 +58,15 @@
 #include "csv.h"
 
 extern gchar *gda_lang_locale;
-
+#if GLIB_CHECK_VERSION(2,31,7)
+static GRecMutex init_rmutex;
+#define MUTEX_LOCK() g_rec_mutex_lock(&init_rmutex)
+#define MUTEX_UNLOCK() g_rec_mutex_unlock(&init_rmutex)
+#else
 static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
+#define MUTEX_LOCK() g_static_rec_mutex_lock(&init_mutex)
+#define MUTEX_UNLOCK() g_static_rec_mutex_unlock(&init_mutex)
+#endif
 static void gda_data_model_class_init (gpointer g_class);
 
 static xmlNodePtr gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
@@ -107,12 +114,12 @@ gda_data_model_get_type (void)
 			0
 		};
 		
-		g_static_rec_mutex_lock (&init_mutex);
+		MUTEX_LOCK();
 		if (type == 0) {
 			type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataModel", &info, 0);
 			g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
 		}
-		g_static_rec_mutex_unlock (&init_mutex);
+		MUTEX_UNLOCK();
 	}
 	return type;
 }
@@ -122,10 +129,10 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 {
 	static gboolean initialized = FALSE;
 
-	g_static_rec_mutex_lock (&init_mutex);
+	MUTEX_LOCK();
 	if (! initialized) {
 		/**
-		 * GdaDataModel::changed
+		 * GdaDataModel::changed:
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when any value in @model has been changed
@@ -139,7 +146,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 		/**
-		 * GdaDataModel::row-inserted
+		 * GdaDataModel::row-inserted:
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -154,7 +161,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::row-updated
+		 * GdaDataModel::row-updated:
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -169,7 +176,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::row-removed
+		 * GdaDataModel::row-removed:
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -184,7 +191,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::reset
+		 * GdaDataModel::reset:
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when @model's contents has been completely reset (the number and
@@ -200,7 +207,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      G_TYPE_NONE, 0);
 
 		/**
-		 * GdaDataModel::access-changed
+		 * GdaDataModel::access-changed:
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when @model's access flags have changed. Use
@@ -217,7 +224,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 
 		initialized = TRUE;
 	}
-	g_static_rec_mutex_unlock (&init_mutex);
+	MUTEX_UNLOCK();
 }
 
 static gboolean
@@ -273,7 +280,7 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 		for (i = 0; i < nbcols; i++) {
 			column = gda_data_model_describe_column (model, i);
 			value = gda_data_model_get_value_at (model, i, 0, NULL);
-			if (value && (gda_column_get_g_type (column) == G_TYPE_INVALID))
+			if (value && (gda_column_get_g_type (column) == GDA_TYPE_NULL))
 				gda_column_set_g_type (column, G_VALUE_TYPE ((GValue *)value));
 		}
 	}
@@ -362,6 +369,7 @@ gda_data_model_reset (GdaDataModel *model)
  * re-enable notifications again, you should call the
  * #gda_data_model_thaw function.
  *
+ * Virtual: i_set_notify
  */
 void
 gda_data_model_freeze (GdaDataModel *model)
@@ -370,8 +378,6 @@ gda_data_model_freeze (GdaDataModel *model)
 	
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify)
 		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, FALSE);
-	else
-		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
 /**
@@ -387,8 +393,6 @@ gda_data_model_thaw (GdaDataModel *model)
 
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify)
 		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, TRUE);
-	else
-		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
 /**
@@ -438,7 +442,7 @@ gda_data_model_get_n_rows (GdaDataModel *model)
  * gda_data_model_get_n_columns:
  * @model: a #GdaDataModel object.
  *
- * Returns: the number of columns in the given data model.
+ * Returns: the number of columns in the given data model, or -1 if unknown.
  *
  * Virtual: i_get_n_columns
  */
@@ -451,7 +455,6 @@ gda_data_model_get_n_columns (GdaDataModel *model)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_columns) (model);
 	else {
 		/* method not supported */
-		g_warning ("%s() method not supported\n", __FUNCTION__);
 		return -1;
 	}
 }
@@ -469,7 +472,7 @@ gda_data_model_get_n_columns (GdaDataModel *model)
  * WARNING: the returned #GdaColumn object belongs to the @model model and
  * and should not be destroyed; any modification will affect the whole data model.
  *
- * Returns: (transfer none): the description of the column.
+ * Returns: (transfer none) (allow-none): the description of the column.
  *
  * Virtual: i_describe_column
  */
@@ -482,7 +485,6 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_describe_column) (model, col);
 	else {
 		/* method not supported */
-		g_warning ("%s() method not supported\n", __FUNCTION__);
 		return NULL;
 	}
 }
@@ -643,10 +645,11 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * Upon errors %NULL will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
- * Returns: (transfer none): a #GValue containing the value stored in the given
+ * Returns: (allow-none) (transfer none): a #GValue containing the value stored in the given
  * position, or %NULL on error (out-of-bound position, etc).
  *
  * Virtual: i_get_value_at
+ *
  */
 const GValue *
 gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
@@ -787,7 +790,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values:
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: (element-type GLib.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
+ * @values: (element-type GObject.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
  * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
@@ -800,6 +803,8 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: %TRUE if the value in the data model has been updated and no error occurred
+ *
+ * Virtual: i_set_values
  */
 gboolean
 gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError **error)
@@ -886,7 +891,7 @@ gda_data_model_create_iter (GdaDataModel *model)
 /**
  * gda_data_model_append_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GLib.Value) (allow-none): #GList of #GValue* representing the row to add.  The
+ * @values: (element-type GObject.Value) (allow-none): #GList of #GValue* representing the row to add.  The
  *          length must match model's column count.  These #GValue
  *	    are value-copied (the user is still responsible for freeing them).
  * @error: a place to store errors, or %NULL
@@ -992,7 +997,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 /**
  * gda_data_model_get_row_from_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GLib.Value): a list of #GValue values (no %NULL is allowed)
+ * @values: (element-type GObject.Value): a list of #GValue values (no %NULL is allowed)
  * @cols_index: (array): an array of #gint containing the column number to match each value of @values
  *
  * Returns the first row where all the values in @values at the columns identified at
@@ -1092,7 +1097,7 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
  * data models where some rows may be missing because the LDAP search has reached a limit
  * imposed by the LDAP server.
  *
- * Returns: (transfer none) (element-type GError) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
+ * Returns: (transfer none) (element-type GLib.Error) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
  *
  * Since: 4.2.6
  */
@@ -1248,7 +1253,9 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 					g_warning (_("The '%s' parameter must hold a boolean value, ignored."), "INVALID_AS_NULL");
 			}
 
-			holder = gda_set_get_holder (options, "FIELDS_NAME");
+			holder = gda_set_get_holder (options, "NAMES_ON_FIRST_LINE");
+			if (!holder)
+				holder = gda_set_get_holder (options, "FIELDS_NAME");
 			if (holder) {
 				const GValue *value;
 				value = gda_holder_get_value (holder);
@@ -1337,12 +1344,12 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * <itemizedlist>
  *   <listitem><para>"SEPARATOR": a string value of which the first character is used as a separator in case of CSV export
  *             </para></listitem>
- *   <listitem><para>"QUOTE": a string value of which the first character is used as a quote character in case of CSV export
- *             </para></listitem>
+ *   <listitem><para>"QUOTE": a string value of which the first character is used as a quote character in case of CSV export. The
+ *             default if not specified is the double quote character</para></listitem>
  *   <listitem><para>"FIELD_QUOTE": a boolean value which can be set to FALSE if no quote around the individual fields 
  *             is requeted, in case of CSV export</para></listitem>
+ *   <listitem><para>"NAMES_ON_FIRST_LINE": a boolean value which, if set to %TRUE and in case of a CSV export, will add a first line with the name each exported field (note that "FIELDS_NAME" is also accepted as a synonym)</para></listitem>
  *   <listitem><para>"NAME": a string value used to name the exported data if the export format is XML</para></listitem>
- *   <listitem><para>"FIELDS_NAME": a boolean value which, if set to %TRUE and in case of a CSV export, will add a first line with the name each exported field</para></listitem>
  *   <listitem><para>"OVERWRITE": a boolean value which tells if the file must be over-written if it already exists.</para></listitem>
  *   <listitem><para>"NULL_AS_EMPTY": a boolean value which, if set to %TRUE and in case of a CSV export, will render and NULL value as the empty string (instead of the 'NULL' string)</para></listitem>
  *   <listitem><para>"INVALID_AS_NULL": a boolean value which, if set to %TRUE, considers any invalid data (for example for the date related values) as NULL</para></listitem>
@@ -1477,9 +1484,15 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols,
 			else if (null_as_empty && gda_value_is_null (value))
 				txt = g_strdup ("");
 			else {
+				gboolean alloc = FALSE;
 				gchar *tmp;
-				
-				tmp = gda_value_stringify (value);
+
+				if (value && (G_VALUE_TYPE (value) == G_TYPE_STRING))
+					tmp = (gchar*) g_value_get_string (value);
+				else {
+					tmp = gda_value_stringify (value);
+					alloc = TRUE;
+				}
 				if (tmp) {
 					gsize len, size;
 					len = strlen (tmp);
@@ -1492,6 +1505,8 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols,
 						txt [len - 1] = 0;
 						memmove (txt, txt+1, len);
 					}
+					if (alloc)
+						g_free (tmp);
 				}
 				else {
 					if (field_quotes) {
@@ -1709,8 +1724,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 		}
 
 		gdatype = gda_column_get_g_type (column);
-		if ((gdatype == G_TYPE_INVALID) ||
-		    (gdatype == GDA_TYPE_NULL)) {
+		if ((gdatype == G_TYPE_INVALID) || (gdatype == GDA_TYPE_NULL)) {
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_XML_FORMAT_ERROR,
 				      "%s", _("Cannot retrieve column data type (type is UNKNOWN or not specified)"));
 			retval = FALSE;
@@ -1729,14 +1743,12 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 			value = g_new0 (GValue, 1);
 			gchar* nodeval = (gchar*)xmlNodeGetContent (xml_field);
 			if (nodeval) {
-				if (!gda_value_set_from_string (value, nodeval, gdatype)) {
-					g_free (value);
-					value = gda_value_new_null ();
-				}
+				if (!gda_value_set_from_string (value, nodeval, gdatype))
+					gda_value_set_null (value);
 				xmlFree(nodeval);
 			}
 			else
-				value = gda_value_new_null ();	
+				gda_value_set_null (value);
 		}
 
 		g_ptr_array_index (values, pos) = value;
@@ -1946,9 +1958,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 
 		column = gda_data_model_describe_column (to, i);
 		if (plist->data) {
-			if ((gda_holder_get_g_type (GDA_HOLDER (plist->data)) != 
-			     gda_column_get_g_type (column)) &&
-			    (gda_column_get_g_type (column) != G_TYPE_INVALID)) {
+			if ((gda_holder_get_g_type (GDA_HOLDER (plist->data)) != gda_column_get_g_type (column)) &&
+			    (gda_column_get_g_type (column) != GDA_TYPE_NULL)) {
 				GValue *newval;
 				
 				newval = g_new0 (GValue, 1);
@@ -2001,7 +2012,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 				value = (GValue *) gda_holder_get_value (GDA_HOLDER (plist->data));
 				if (avlist->data) {
 					if (append_types [i] && gda_value_is_null ((GValue *) (avlist->data))) 
-						g_value_init ((GValue *) (avlist->data), append_types [i]);
+						gda_value_reset_with_type ((GValue *) (avlist->data), append_types [i]);
 					if (!gda_value_is_null (value) && 
 					    !g_value_transform (value, (GValue *) (avlist->data))) {
 						gchar *str;
@@ -2346,16 +2357,14 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 #ifdef HAVE_LOCALE_H
 #ifndef G_OS_WIN32
-	char *current_locale;
 	int utf8_mode;
-	current_locale = setlocale (LC_ALL, "");
-	utf8_mode = (strcmp (nl_langinfo(CODESET), "UTF-8") == 0);
+	setlocale (LC_ALL, NULL);
+	utf8_mode = (strcmp (nl_langinfo (CODESET), "UTF-8") == 0);
 	if (utf8_mode) {
 		sep_col = " │ ";
 		sep_fill = "─";
 		sep_row = "─┼─";
 	}
-	setlocale (LC_ALL, current_locale);
 #endif
 #endif
 
@@ -2421,18 +2430,30 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 					cols_size [i + col_offset] = MAX ((guint)cols_size [i + col_offset], strlen (ERROR_STRING));
 				}
 				else {
+					gboolean alloc = FALSE;
 					str = NULL;
 					if (null_as_empty) {
 						if (!value || gda_value_is_null (value))
-							str = g_strdup ("");
+							str = "";
+					}
+					if (!str) {
+						if (value) {
+							if (G_VALUE_TYPE (value) == G_TYPE_STRING)
+								str = (gchar*) g_value_get_string (value);
+							else {
+								str = gda_value_stringify ((GValue*)value);
+								alloc = TRUE;
+							}
+						}
+						else
+							str = "_null_";
 					}
-					if (!str)
-						str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
 					if (str) {
 						gint w;
 						string_get_dimensions (str, &w, NULL);
 						cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
-						g_free (str);
+						if (alloc)
+							g_free (str);
 					}
 				}
 			}
@@ -2527,28 +2548,41 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 		}
 		
 		for (i = 0; i < n_cols; i++) {
+			gboolean alloc = FALSE;
+			str = NULL;
 			if (!dump_attributes) {
 				value = gda_data_model_get_value_at (ramodel, i, j, NULL);
 				if (!value)
-					str = g_strdup (ERROR_STRING);
+					str = ERROR_STRING;
 				else {
-					str = NULL;
 					if (null_as_empty) {
 						if (!value || gda_value_is_null (value))
-							str = g_strdup ("");
+							str = "";
+					}
+					if (!str) {
+						if (value) {
+							if (G_VALUE_TYPE (value) == G_TYPE_STRING)
+								str = (gchar*) g_value_get_string (value);
+							else {
+								str = gda_value_stringify ((GValue*)value);
+								alloc = TRUE;
+							}
+						}
+						else
+							str = "_null_";
 					}
-					if (!str)
-						str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
 				}
 			}
 			else {
 				GdaValueAttribute attrs;
 				attrs = gda_data_model_get_attributes_at (ramodel, i, j);
 				str = g_strdup_printf ("%u", attrs);
+				alloc = TRUE;
 			}
 			if (str) {
 				cols_str [i + col_offset] = g_strsplit (str, "\n", -1);
-				g_free (str);
+				if (alloc)
+					g_free (str);
 				cols_height [i + col_offset] = g_strv_length (cols_str [i + col_offset]);
 				kmax = MAX (kmax, cols_height [i + col_offset]);
 			}
