@@ -146,7 +146,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 
 	/* signals */
 	/**
-	 * GdaServerOperation::sequence-item-added
+	 * GdaServerOperation::sequence-item-added:
 	 * @op: the #GdaServerOperation
 	 * @seq_path: the path to the new sequence item
 	 * @item_index: the index (starting from 0) of the new sequence item in the sequence
@@ -162,7 +162,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      _gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
 	/**
-	 * GdaServerOperation::sequence-item-remove
+	 * GdaServerOperation::sequence-item-remove:
 	 * @op: the #GdaServerOperation
 	 * @seq_path: the path to the sequence item to be removed
 	 * @item_index: the index (starting from 0) of the sequence item in the sequence
@@ -1495,7 +1495,8 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 
 	/* actual data loading */
 	if (strcmp ((gchar*)node->name, "serv_op_data")) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, GDA_SERVER_OPERATION_ERROR,
+			     GDA_SERVER_OPERATION_XML_ERROR,
 			     _("Expected tag <%s>, got <%s>"), "serv_op_data", node->name);
 		return FALSE;
 	}
@@ -1509,7 +1510,8 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 		}
 
 		if (strcmp ((gchar*)cur->name, "op_data")) {
-			g_set_error (error, 0, 0,
+			g_set_error (error, GDA_SERVER_OPERATION_ERROR,
+				     GDA_SERVER_OPERATION_XML_ERROR,
 				     _("Expected tag <%s>, got <%s>"), "op_data", cur->name);
 			return FALSE;
 		}
@@ -1540,7 +1542,10 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 				switch (opnode->type) {
 				case GDA_SERVER_OPERATION_NODE_PARAMLIST:
 					if (!extension) {
-						g_set_error (error, 0, 0, "%s", 
+						g_set_error (error,
+							     GDA_SERVER_OPERATION_ERROR,
+							     GDA_SERVER_OPERATION_XML_ERROR,
+							     "%s", 
 							     _("Parameterlist values can only be set for individual parameters within it"));
 						allok = FALSE;
 					}
@@ -1597,8 +1602,9 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 				return FALSE;
 		}
 		else {
-			g_set_error (error, 0, 0, "%s", 
-				     _("Missing attribute named 'path'"));
+			g_set_error (error, GDA_SERVER_OPERATION_ERROR,
+				     GDA_SERVER_OPERATION_XML_ERROR,
+				     "%s", _("Missing attribute named 'path'"));
 			return FALSE;
 		}
 		
@@ -2256,7 +2262,9 @@ gda_server_operation_set_value_at_path (GdaServerOperation *op, const gchar *val
 		switch (opnode->type) {
 		case GDA_SERVER_OPERATION_NODE_PARAMLIST:
 			if (!extension) {
-				g_set_error (error, 0, 0, "%s", 
+				g_set_error (error, GDA_SERVER_OPERATION_ERROR,
+					     GDA_SERVER_OPERATION_XML_ERROR,
+					     "%s", 
 					     _("Parameterlist values can only be set for individual parameters within it"));
 				allok = FALSE;
 			}
@@ -2446,7 +2454,9 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 					value = gda_server_operation_get_value_at (op, path);
 					if (!value) {
 						valid = FALSE;
-						g_set_error (error, 0, 0,
+						g_set_error (error,
+							     GDA_SERVER_OPERATION_ERROR,
+							     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
 							     _("Missing required value for '%s'"), path);
 						break;
 					}
@@ -2615,6 +2625,7 @@ gda_server_operation_perform_drop_database (GdaServerOperation *op, const gchar
 	GdaServerProvider *prov;
 
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+	g_return_val_if_fail (gda_server_operation_get_op_type (op) == GDA_SERVER_OPERATION_DROP_DB, FALSE);
 	if (provider)
 		prov = gda_config_get_provider (provider, error);
 	else
