@@ -75,7 +75,8 @@ enum
 	PROP_ID,
 	PROP_NAME,
 	PROP_DESCR,
-	PROP_HOLDERS
+	PROP_HOLDERS,
+	PROP_VALIDATE_CHANGES
 };
 
 /* signals */
@@ -103,6 +104,7 @@ struct _GdaSetPrivate
 	GHashTable      *holders_hash; /* key = GdaHoler ID, value = GdaHolder */
 	GArray          *holders_array;
 	gboolean         read_only;
+	gboolean         validate_changes;
 };
 
 static void 
@@ -130,11 +132,29 @@ gda_set_set_property (GObject *object,
 	case PROP_HOLDERS: {
 		/* add the holders */
 		GSList* holders;
-		for (holders = (GSList*) g_value_get_pointer(value); holders; holders = holders->next) 
+		for (holders = (GSList*) g_value_get_pointer (value); holders; holders = holders->next) 
 			gda_set_real_add_holder (set, GDA_HOLDER (holders->data));
 		compute_public_data (set);	
 		break;
 	}
+	case PROP_VALIDATE_CHANGES:
+		if (set->priv->validate_changes != g_value_get_boolean (value)) {
+			GSList *list;
+			set->priv->validate_changes = g_value_get_boolean (value);
+			for (list = set->holders; list; list = list->next) {
+				GdaHolder *holder = (GdaHolder*) list->data;
+				g_object_set ((GObject*) holder, "validate-changes",
+					      set->priv->validate_changes, NULL);
+				if (set->priv->validate_changes)
+					g_signal_connect ((GObject*) holder, "validate-change",
+							  G_CALLBACK (validate_change_holder_cb), set);
+				else
+					g_signal_handlers_disconnect_by_func ((GObject*) holder,
+									      G_CALLBACK (validate_change_holder_cb),
+									      set);
+			}
+		}
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -163,6 +183,9 @@ gda_set_get_property (GObject *object,
 	case PROP_DESCR:
 		g_value_set_string (value, set->priv->descr);
 		break;
+	case PROP_VALIDATE_CHANGES:
+		g_value_set_boolean (value, set->priv->validate_changes);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -252,7 +275,7 @@ gda_set_class_init (GdaSetClass *class)
 			      GDA_TYPE_HOLDER);
 
 	/**
-	 * GdaSet::validate-holder-change
+	 * GdaSet::validate-holder-change:
 	 * @set: the #GdaSet
 	 * @holder: the #GdaHolder which is going to change
 	 * @new_value: the proposed new value for @holder
@@ -272,7 +295,7 @@ gda_set_class_init (GdaSetClass *class)
 			      _gda_marshal_ERROR__OBJECT_VALUE, G_TYPE_ERROR, 2,
 			      GDA_TYPE_HOLDER, G_TYPE_VALUE);
 	/**
-	 * GdaSet::validate-set
+	 * GdaSet::validate-set:
 	 * @set: the #GdaSet
 	 * 
 	 * Gets emitted when gda_set_is_valid() is called, use
@@ -289,7 +312,7 @@ gda_set_class_init (GdaSetClass *class)
 			      validate_accumulator, NULL,
 			      _gda_marshal_ERROR__VOID, G_TYPE_ERROR, 0);
 	/**
-	 * GdaSet::holder-attr-changed
+	 * GdaSet::holder-attr-changed:
 	 * @set: the #GdaSet
 	 * @holder: the GdaHolder for which an attribute changed
 	 * @attr_name: attribute's name
@@ -306,7 +329,7 @@ gda_set_class_init (GdaSetClass *class)
 			      _gda_marshal_VOID__OBJECT_STRING_VALUE, G_TYPE_NONE, 3,
 			      GDA_TYPE_HOLDER, G_TYPE_STRING, G_TYPE_VALUE);
 	/**
-	 * GdaSet::public-data-changed
+	 * GdaSet::public-data-changed:
 	 * @set: the #GdaSet
 	 * 
 	 * Gets emitted when @set's public data (#GdaSetNode, #GdaSetGroup or #GdaSetSource values) have changed
@@ -320,7 +343,7 @@ gda_set_class_init (GdaSetClass *class)
 			      _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	/**
-	 * GdaSet::holder-type-set
+	 * GdaSet::holder-type-set:
 	 * @set: the #GdaSet
 	 * @holder: the #GdaHolder for which the #GType has been set
 	 *
@@ -339,7 +362,7 @@ gda_set_class_init (GdaSetClass *class)
 			      GDA_TYPE_HOLDER);
 
 	/**
-	 * GdaSet::source-model-changed
+	 * GdaSet::source-model-changed:
 	 * @set: the #GdaSet
 	 * @source: the #GdaSetSource for which the @data_model attribute has changed
 	 *
@@ -381,7 +404,18 @@ gda_set_class_init (GdaSetClass *class)
 					 g_param_spec_pointer ("holders", "GSList of GdaHolders", 
 							       "GdaHolder objects the set should contain", (
 								G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
-  
+	/**
+	 * GdaSet:validate-changes:
+	 *
+	 * Defines if the "validate-set" signal gets emitted when
+	 * any holder in the data set changes. This property also affects the
+	 * GdaHolder:validate-changes property.
+	 *
+	 * Since: 5.2.0
+	 */
+	g_object_class_install_property (object_class, PROP_VALIDATE_CHANGES,
+					 g_param_spec_boolean ("validate-changes", NULL, "Defines if the validate-set signal is emitted", TRUE,
+							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	object_class->dispose = gda_set_dispose;
 	object_class->finalize = gda_set_finalize;
 }
@@ -397,12 +431,13 @@ gda_set_init (GdaSet *set)
 	set->priv->holders_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	set->priv->holders_array = NULL;
 	set->priv->read_only = FALSE;
+	set->priv->validate_changes = TRUE;
 }
 
 
 /**
  * gda_set_new:
- * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
+ * @holders: (element-type Gda.Holder) (transfer none): a list of #GdaHolder objects
  *
  * Creates a new #GdaSet object, and populates it with the list given as argument.
  * The list can then be freed as it is copied. All the value holders in @holders are referenced counted
@@ -425,7 +460,7 @@ gda_set_new (GSList *holders)
 
 /**
  * gda_set_new_read_only:
- * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
+ * @holders: (element-type Gda.Holder) (transfer none): a list of #GdaHolder objects
  *
  * Creates a new #GdaSet like gda_set_new(), but does not allow modifications to any of the #GdaHolder
  * object in @holders. This function is used for Libgda's database providers' implementation.
@@ -533,7 +568,11 @@ gda_set_new_inline (gint nb, ...)
 		else if (type == GDA_TYPE_USHORT)
 			gda_value_set_ushort (value, va_arg (ap, guint));
 		else if (type == G_TYPE_CHAR)
+#if GLIB_CHECK_VERSION(2,31,7)
+			g_value_set_schar (value, va_arg (ap, gint));
+#else
 			g_value_set_char (value, va_arg (ap, int));
+#endif
 		else if (type == G_TYPE_UCHAR)
 			g_value_set_uchar (value, va_arg (ap, guint));
 		else if (type == G_TYPE_FLOAT)
@@ -637,7 +676,11 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
 	else if (type == GDA_TYPE_USHORT)
 		gda_value_set_ushort (value, va_arg (ap, guint));
 	else if (type == G_TYPE_CHAR)
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_value_set_schar (value, va_arg (ap, gint));
+#else
 		g_value_set_char (value, va_arg (ap, int));
+#endif
 	else if (type == G_TYPE_UCHAR)
 		g_value_set_uchar (value, va_arg (ap, guint));
 	else if (type == G_TYPE_FLOAT)
@@ -655,7 +698,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
 	else if (type == G_TYPE_GTYPE)
 		g_value_set_gtype (value, va_arg (ap, GType));	
 	else {
-		g_set_error (error, 0, 0,
+		g_set_error (error, GDA_SET_ERROR, GDA_SET_IMPLEMENTATION_ERROR,
 			     _("%s() does not handle values of type '%s'."),
 			     __FUNCTION__, g_type_name (type));
 		va_end (ap);
@@ -850,7 +893,8 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 				errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (model));
 				if (errors) {
 					GError *err = (GError *) errors->data;
-					g_set_error (error, 0, 0, "%s", err->message);
+					g_set_error (error, GDA_SET_ERROR, GDA_SET_XML_SPEC_ERROR,
+						     "%s", err->message);
 					g_object_unref (model);
 					model = NULL;
 					allok = FALSE;
@@ -984,8 +1028,9 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 	g_return_if_fail (set->priv);
 	g_return_if_fail (g_slist_find (set->holders, holder));
 
-	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-					      G_CALLBACK (validate_change_holder_cb), set);
+	if (set->priv->validate_changes)
+		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+						      G_CALLBACK (validate_change_holder_cb), set);
 	if (! set->priv->read_only) {
 		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 						      G_CALLBACK (changed_holder_cb), set);
@@ -1044,7 +1089,7 @@ validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *set)
 	/* signal the holder validate-change */
 	GError *error = NULL;
 	if (set->priv->read_only)
-		g_set_error (&error, GDA_SET_ERROR, GDA_SET_READ_ONLY_ERROR, _("Data set does not allow modifications"));
+		g_set_error (&error, GDA_SET_ERROR, GDA_SET_READ_ONLY_ERROR, "%s", _("Data set does not allow modifications"));
 	else {
 #ifdef GDA_DEBUG_signal
 		g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
@@ -1090,8 +1135,9 @@ gda_set_dispose (GObject *object)
 	/* free the holders list */
 	if (set->holders) {
 		for (list = set->holders; list; list = list->next) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-							      G_CALLBACK (validate_change_holder_cb), set);
+			if (set->priv->validate_changes)
+				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+								      G_CALLBACK (validate_change_holder_cb), set);
 			if (! set->priv->read_only) {
 				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 								      G_CALLBACK (changed_holder_cb), set);
@@ -1323,8 +1369,9 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 			set->priv->holders_array = NULL;
 		}
 		g_object_ref (holder);
-		g_signal_connect (G_OBJECT (holder), "validate-change",
-				  G_CALLBACK (validate_change_holder_cb), set);
+		if (set->priv->validate_changes)
+			g_signal_connect (G_OBJECT (holder), "validate-change",
+					  G_CALLBACK (validate_change_holder_cb), set);
 		if (! set->priv->read_only) {
 			g_signal_connect (G_OBJECT (holder), "changed",
 					  G_CALLBACK (changed_holder_cb), set);
@@ -1615,7 +1662,7 @@ gda_set_get_source_for_model (GdaSet *set, GdaDataModel *model)
 }
 
 /**
- * gda_set_replace_source_model
+ * gda_set_replace_source_model:
  * @set: a #GdaSet object
  * @source: a pointer to a #GdaSetSource in @set
  * @model: a #GdaDataModel
