@@ -1,23 +1,21 @@
 /*
- * Copyright (C) 2009 - 2011 The GNOME Foundation.
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -99,7 +97,7 @@ gda_tree_node_class_init (GdaTreeNodeClass *klass)
 
 	/* signals */
 	/**
-	 * GdaTreeNode::node-changed
+	 * GdaTreeNode::node-changed:
 	 * @reporting: the #GdaTreeNode which emits the signal (may be a parent of @node, or @node itself)
 	 * @node: the #GdaTreeNode which has changed
 	 *
@@ -116,7 +114,7 @@ gda_tree_node_class_init (GdaTreeNodeClass *klass)
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1, GDA_TYPE_TREE_NODE);
 	/**
-	 * GdaTreeNode::node-inserted
+	 * GdaTreeNode::node-inserted:
 	 * @reporting: the #GdaTreeNode which emits the signal (may be a parent of @node, or @node itself)
 	 * @node: the #GdaTreeNode which has been inserted
 	 *
@@ -133,7 +131,7 @@ gda_tree_node_class_init (GdaTreeNodeClass *klass)
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1, GDA_TYPE_TREE_NODE);
 	/**
-	 * GdaTreeNode::node-has-child-toggled
+	 * GdaTreeNode::node-has-child-toggled:
 	 * @reporting: the #GdaTreeNode which emits the signal (may be a parent of @node, or @node itself)
 	 * @node: the #GdaTreeNode which changed from having children to being a
 	 *        leaf or the other way around
@@ -152,7 +150,7 @@ gda_tree_node_class_init (GdaTreeNodeClass *klass)
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1, GDA_TYPE_TREE_NODE);
 	/**
-	 * GdaTreeNode::node-deleted
+	 * GdaTreeNode::node-deleted:
 	 * @reporting: the #GdaTreeNode which emits the signal (a parent of the removed node)
 	 * @relative_path: the path the node held, relative to @reporting
 	 *
@@ -458,7 +456,7 @@ gda_tree_node_dump_children (GdaTreeNode *node, const gchar *prefix, GString *in
 
 /**
  * gda_tree_node_new:
- * @name: a name, or %NULL
+ * @name: (allow-none): a name, or %NULL
  *
  * Creates a new #GdaTreeNode object
  *
@@ -476,7 +474,7 @@ gda_tree_node_new (const gchar *name)
  * _gda_tree_node_add_children:
  * @node: a #GdaTreeNode
  * @mgr: a #GdaTreeManager object
- * @childen: a list of #GdaTreeNode objects, or %NULL
+ * @childen: (allow-none): a list of #GdaTreeNode objects, or %NULL
  *
  * Requests that @node take the list of nodes in @children as its children. The children list
  * may be a new list of children, or may be an update of a list of children which previously existed.
@@ -622,12 +620,11 @@ _gda_tree_node_add_children (GdaTreeNode *node, GdaTreeManager *mgr, const GSLis
 const GSList *
 _gda_tree_node_get_children_for_manager (GdaTreeNode *node, GdaTreeManager *mgr)
 {
-	GdaTreeNodesList *etn;
 	GSList *list;
 	g_return_val_if_fail (GDA_IS_TREE_NODE (node), NULL);
 	g_return_val_if_fail (GDA_IS_TREE_MANAGER (mgr), NULL);
 	
-	for (etn = NULL, list = node->priv->children; list; list = list->next) {
+	for (list = node->priv->children; list; list = list->next) {
 		if (GDA_TREE_NODES_LIST (list->data)->mgr == mgr)
 			return GDA_TREE_NODES_LIST (list->data)->nodes;
 	}
@@ -722,7 +719,7 @@ gda_tree_node_get_node_attribute (GdaTreeNode *node, const gchar *attribute)
  * gda_tree_node_set_node_attribute:
  * @node: a #GdaTreeNode
  * @attribute: attribute name
- * @value: (transfer none): a #GValue, or %NULL
+ * @value: (transfer none) (allow-none): a #GValue, or %NULL
  * @destroy: a function to be called when @attribute is not needed anymore, or %NULL
  *
  * Set the value associated to a named attribute. The @attribute string is used AS IT IS by this method (eg.
