@@ -51,6 +51,7 @@
 #include <sql-parser/gda-statement-struct-util.h>
 #include <libgda/gda-set.h>
 #include <libgda/gda-blob-op.h>
+#include <libgda/gda-statement-priv.h>
 
 #include <libgda/binreloc/gda-binreloc.h>
 
@@ -171,6 +172,10 @@ gda_g_type_from_string (const gchar *str)
 			type = G_TYPE_CHAR;
 		else if (!g_ascii_strcasecmp (str, "uchar"))
 			type = G_TYPE_UCHAR;
+		else if (!g_ascii_strcasecmp (str, "gshort"))
+			type = GDA_TYPE_SHORT;
+		else if (!g_ascii_strcasecmp (str, "gushort"))
+			type = GDA_TYPE_USHORT;
 		else
 			/* could not find a valid GType for @str */
 			type = G_TYPE_INVALID;
@@ -1512,7 +1517,7 @@ null_param_unknown_foreach_func (GdaSqlAnyPart *part, NullData *data, GError **e
 }
 
 /**
- * gda_rewrite_sql_statement_for_null_parameters: (skip):
+ * gda_rewrite_sql_statement_for_null_parameters: (skip)
  * @sqlst: (transfer full): a #GdaSqlStatement
  * @params: a #GdaSet to be used as parameters when executing @stmt
  * @out_modified: (allow-none): a place to store the boolean which tells if @stmt has been modified or not, or %NULL
@@ -1721,7 +1726,7 @@ gda_statement_rewrite_for_default_values (GdaStatement *stmt, GdaSet *params, gb
 		if (remove)
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
-				     _("Can't rewrite UPDATE statement to handle default values"));
+				     "%s", _("Can't rewrite UPDATE statement to handle default values"));
 		else
 			ok = stmt_rewrite_update_default_keyword ((GdaSqlStatementUpdate*) sqlst->contents,
 								  params, error);
@@ -1729,7 +1734,7 @@ gda_statement_rewrite_for_default_values (GdaStatement *stmt, GdaSet *params, gb
 	default:
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
-			     "Can't rewrite statement is not INSERT or UPDATE");
+			     "%s", _("Can't rewrite statement which is not INSERT or UPDATE"));
 		break;
 	}
 
@@ -1756,7 +1761,7 @@ stmt_rewrite_insert_remove (GdaSqlStatementInsert *ins, GdaSet *params, GError *
 		TO_IMPLEMENT;
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_DEFAULT_VALUE_HANDLING_ERROR,
-			     "Not yet implemented");
+			     "%s", "Not yet implemented");
 		return FALSE;
 	}
 
@@ -1892,6 +1897,62 @@ stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd, GdaSet *params,
 	return TRUE;
 }
 
+
+static gboolean
+foreach_modify_param_type (GdaSqlAnyPart *part, GdaDataModel *model, G_GNUC_UNUSED GError **error)
+{
+	if (part->type != GDA_SQL_ANY_EXPR)
+		return TRUE;
+
+	GdaSqlParamSpec *pspec;
+	pspec = ((GdaSqlExpr*) part)->param_spec;
+	if (!pspec || !pspec->name)
+		return TRUE;
+
+	if ((pspec->name [0] == '+') || (pspec->name [0] == '-')) {
+		long int li;
+		char *end;
+		li = strtol ((pspec->name) + 1, &end, 10);
+		if ((! *end) && (li <= G_MAXINT) && (li >= G_MININT) && (li < gda_data_model_get_n_columns (model)))  {
+			GdaColumn *col;
+			col = gda_data_model_describe_column (model, (gint) li);
+			if (col && (gda_column_get_g_type (col) != GDA_TYPE_NULL))
+				pspec->g_type = gda_column_get_g_type (col);
+		}
+	}
+	return TRUE;
+}
+
+/**
+ * gda_modify_statement_param_types:
+ * @stmt: a #GdaStatement
+ * @model: a #GdaDataModel
+ *
+ * Modifies the parameters in @stmt which will be mapped to columns in @model (using the +&lt;colindex&gt; or
+ * -&lt;colindex&gt; syntax) to map the column types of @model.
+ *
+ * Since: 5.2
+ */
+void
+_gda_modify_statement_param_types (GdaStatement *stmt, GdaDataModel *model)
+{
+	g_return_if_fail (GDA_IS_STATEMENT (stmt));
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+	GdaSqlStatement *sqlst;
+
+	sqlst = _gda_statement_get_internal_struct (stmt);
+	if (!sqlst || !sqlst->contents)
+		return;
+
+	if ((sqlst->stmt_type == GDA_SQL_STATEMENT_INSERT) ||
+	    (sqlst->stmt_type == GDA_SQL_STATEMENT_UPDATE) ||
+	    (sqlst->stmt_type == GDA_SQL_STATEMENT_DELETE)) {
+		GdaSqlAnyPart *top;
+		top = (GdaSqlAnyPart*) sqlst->contents;
+		gda_sql_any_part_foreach (top, (GdaSqlForeachFunc) foreach_modify_param_type, model, NULL);
+	}
+}
+
 /**
  * gda_identifier_hash:
  * @id: an identifier string
