@@ -39,6 +39,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
+#include <locale.h>
 #include <libgda/gda-value.h>
 #include <libgda/gda-blob-op.h>
 #include <libgda/gda-util.h>
@@ -52,6 +53,36 @@
 #define bcmp(s1, s2, n) memcmp ((s1), (s2), (n))
 #endif
 
+extern gchar *gda_numeric_locale;
+
+#  ifdef GSEAL_ENABLE
+/**
+ * GdaNumeric:
+ * @number: a string representing a number
+ * @precision: precision to use when @number is converted (not implemented jet)
+ * @width: No implemented jet
+ *
+ * Holds numbers represented as strings.
+ *
+ * This struct must be considered as opaque. Any access to its members must use its
+ * accessors added since version 5.0.2.
+ *
+ * Set value func: gda_value_set_numeric
+ * Get value func: gda_value_get_numeric
+ */
+struct _GdaNumeric {
+	gchar*   number;
+	glong    precision;
+	glong    width;
+	
+	/*< private >*/
+	gpointer reserved; /* reserved for future usage with GMP (http://gmplib.org/) */
+};
+#  else
+#endif
+
+
+
 static gboolean
 set_from_string (GValue *value, const gchar *as_string)
 {
@@ -140,7 +171,11 @@ set_from_string (GValue *value, const gchar *as_string)
 	else if (type == G_TYPE_CHAR) {
 		lvalue = strtol (as_string, endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
+#if GLIB_CHECK_VERSION(2,31,7)
+			g_value_set_schar(value,(gint)lvalue);
+#else
 			g_value_set_char(value,(gchar)lvalue);
+#endif
 			retval = TRUE;
 		}
 	}
@@ -166,13 +201,10 @@ set_from_string (GValue *value, const gchar *as_string)
 		}
 	}
 	else if (type == GDA_TYPE_NUMERIC) {
-		GdaNumeric numeric;
-		/* FIXME: what test whould i do for numeric? Use GMP (http://gmplib.org/) ?*/
-		numeric.number = g_strdup (as_string);
-		numeric.precision = 0; /* FIXME */
-		numeric.width = 0; /* FIXME */
-		gda_value_set_numeric (value, &numeric);
-		g_free (numeric.number);
+		GdaNumeric *numeric = gda_numeric_new ();
+		gda_numeric_set_from_string (numeric, as_string);
+		gda_value_set_numeric (value, numeric);
+		gda_numeric_free (numeric);
 		retval = TRUE;
 	}
 	else if (type == G_TYPE_DATE) {
@@ -205,8 +237,10 @@ set_from_string (GValue *value, const gchar *as_string)
 		retval = TRUE;
 	}
 	else if (type == G_TYPE_GTYPE) {
-		if (gda_g_type_from_string (as_string) != 0) {
-			g_value_set_gtype (value, gda_g_type_from_string (as_string));
+		GType gt;
+		gt = gda_g_type_from_string (as_string);
+		if (gt != G_TYPE_INVALID) {
+			g_value_set_gtype (value, gt);
 			retval = TRUE;
 		}
 	}
@@ -315,6 +349,63 @@ gda_null_get_type (void)
        return type;
 }
 
+/*
+ * Register the DEFAULT type in the GType system
+ */
+static void
+string_to_default (const GValue *src, GValue *dest)
+{
+       g_return_if_fail (G_VALUE_HOLDS_STRING (src) && GDA_VALUE_HOLDS_DEFAULT (dest));
+       g_value_set_boxed (dest, g_value_get_string (src));
+}
+
+static void
+default_to_string (const GValue *src, GValue *dest)
+{
+	gchar *str;
+	g_return_if_fail (G_VALUE_HOLDS_STRING (dest) && GDA_VALUE_HOLDS_DEFAULT (src));
+	str = (gchar*) g_value_get_boxed (src);
+	g_value_set_string (dest, str);
+}
+
+static gpointer
+gda_default_copy (G_GNUC_UNUSED gpointer boxed)
+{
+	if (boxed)
+		return (gpointer) g_strdup (boxed);
+	else
+		return (gpointer) NULL;
+}
+
+static void
+gda_default_free (G_GNUC_UNUSED gpointer boxed)
+{
+	g_free (boxed);
+}
+
+GType
+gda_default_get_type (void)
+{
+       static GType type = 0;
+
+       if (G_UNLIKELY (type == 0)) {
+               type = g_boxed_type_register_static ("GdaDefault",
+                                                    (GBoxedCopyFunc) gda_default_copy,
+                                                    (GBoxedFreeFunc) gda_default_free);
+
+               g_value_register_transform_func (G_TYPE_STRING,
+                                                type,
+                                                string_to_default);
+
+               g_value_register_transform_func (type,
+                                                G_TYPE_STRING,
+                                                default_to_string);
+       }
+
+       return type;
+}
+
+
 /*
  * Register the GdaBinary type in the GType system
  */
@@ -521,7 +612,7 @@ gda_blob_free (gpointer boxed)
 /**
  * gda_blob_set_op:
  * @blob: a #GdaBlob value
- * @op: a #GdaBlobOp object, or %NULL
+ * @op: (allow-none): a #GdaBlobOp object, or %NULL
  *
  * correctly assigns @op to @blob
  */
@@ -707,6 +798,36 @@ numeric_to_boolean (const GValue *src, GValue *dest)
 		g_value_set_boolean (dest, 0);
 }
 
+static void
+numeric_to_double (const GValue *src, GValue *dest)
+{
+	const GdaNumeric *numeric;
+
+	g_return_if_fail (G_VALUE_HOLDS_DOUBLE (dest) &&
+			  GDA_VALUE_HOLDS_NUMERIC (src));
+
+	numeric = gda_value_get_numeric (src);
+	if (numeric)
+		g_value_set_double (dest, g_strtod (numeric->number, NULL));
+	else
+		g_value_set_double (dest, 0.0);
+}
+
+static void
+numeric_to_float (const GValue *src, GValue *dest)
+{
+	const GdaNumeric *numeric;
+
+	g_return_if_fail (G_VALUE_HOLDS_FLOAT (dest) &&
+			  GDA_VALUE_HOLDS_NUMERIC (src));
+
+	numeric = gda_value_get_numeric (src);
+	if (numeric)
+		g_value_set_float (dest, (float) g_strtod (numeric->number, NULL));
+	else
+		g_value_set_float (dest, 0.0);
+}
+
 GType
 gda_numeric_get_type (void)
 {
@@ -723,14 +844,17 @@ gda_numeric_get_type (void)
 		g_value_register_transform_func (type, G_TYPE_INT, numeric_to_int);
 		g_value_register_transform_func (type, G_TYPE_UINT, numeric_to_uint);
 		g_value_register_transform_func (type, G_TYPE_BOOLEAN, numeric_to_boolean);
+		g_value_register_transform_func (type, G_TYPE_DOUBLE, numeric_to_double);
+		g_value_register_transform_func (type, G_TYPE_FLOAT, numeric_to_float);
 	}
 
 	return type;
 }
 
+
 /**
  * gda_numeric_copy:
- * @boxed: source to get a copy from.
+ * @src: source to get a copy from.
  *
  * Creates a new #GdaNumeric structure from an existing one.
 
@@ -739,39 +863,191 @@ gda_numeric_get_type (void)
  * Free-function: gda_numeric_free
  */
 
-gpointer
-gda_numeric_copy (gpointer boxed)
+GdaNumeric*
+gda_numeric_copy (GdaNumeric *src)
 {
-	GdaNumeric *src = (GdaNumeric*) boxed;
-	GdaNumeric *copy = NULL;
+	GdaNumeric *copy;
 
 	g_return_val_if_fail (src, NULL);
 
-	copy = g_new0 (GdaNumeric, 1);
-	copy->number = g_strdup (src->number);
-	copy->precision = src->precision;
-	copy->width = src->width;
-
+	copy = gda_numeric_new ();
+	gda_numeric_set_from_string (copy, gda_numeric_get_string (src));
+	gda_numeric_set_width (copy, gda_numeric_get_width (src));
+	gda_numeric_set_precision (copy, gda_numeric_get_precision (src));
 	return copy;
 }
 
 /**
  * gda_numeric_free:
- * @boxed: (transfer full): a #GdaNumeric pointer
+ * @numeric: (transfer full): a #GdaNumeric pointer
  *
  * Deallocates all memory associated to the given @boxed
  */
 void
-gda_numeric_free (gpointer boxed)
+gda_numeric_free (GdaNumeric *numeric)
 {
-	GdaNumeric *numeric = (GdaNumeric*) boxed;
 	g_return_if_fail (numeric);
 
 	g_free (numeric->number);
 	g_free (numeric);
 }
 
+/**
+ * gda_numeric_new:
+ *
+ * Creates a new #GdaNumeric with defaults.
+ *
+ * Returns: (transfer full): a new #GdaNumeric.
+ * Since: 5.0.2
+ */
+GdaNumeric*
+gda_numeric_new (void)
+{
+	GdaNumeric *n = g_new0 (GdaNumeric, 1);
+	setlocale (LC_NUMERIC, "C");
+	n->number = g_strdup_printf ("%lf", 0.0);
+	setlocale (LC_NUMERIC, gda_numeric_locale);
+	return n;
+}
 
+/**
+ * gda_numeric_set_from_string:
+ * @numeric: a #GdaNumeric
+ * @str: a string representing a number
+ *
+ * Sets @numeric with a number represented by @str. By default converts @str to
+ * #gdouble.
+ *
+ * * Since: 5.0.2
+ */
+void
+gda_numeric_set_from_string (GdaNumeric *numeric, const gchar* str)
+{
+	g_return_if_fail (numeric);
+	g_return_if_fail (str);
+	g_free (numeric->number);
+	// FIXME: By default convert string to gdouble, for other number types we need to check string format
+	gdouble n = g_strtod (str, NULL);
+	setlocale (LC_NUMERIC, "C");
+	numeric->number = g_strdup_printf ("%lf", n);
+	setlocale (LC_NUMERIC, gda_numeric_locale);
+}
+
+
+/**
+ * gda_numeric_set_double:
+ * @numeric: a #GdaNumeric
+ * @number: a #gdouble
+ *
+ * Sets @numeric using a #gdouble represented by @number.
+ *
+ * Since: 5.0.2
+ */
+void
+gda_numeric_set_double (GdaNumeric *numeric, gdouble number)
+{
+	g_return_if_fail (numeric);
+	g_free (numeric->number);
+	setlocale (LC_NUMERIC, "C");
+	numeric->number = g_strdup_printf ("%lf", number);
+	setlocale (LC_NUMERIC, gda_numeric_locale);
+}
+
+/**
+ * gda_numeric_get_double:
+ * @numeric: a #GdaNumeric
+ *
+ * Returns: a #gdouble representation of @numeric
+ * Since: 5.0.2
+ */
+gdouble
+gda_numeric_get_double (GdaNumeric *numeric)
+{
+	g_return_val_if_fail (numeric, 0.0);
+	if (numeric->number)
+		return atof (numeric->number);
+	else
+		return 0.0;
+}
+
+/**
+ * gda_numeric_set_width:
+ * @numeric: a #GdaNumeric
+ * @width: a #glong
+ *
+ * Sets the width of a #GdaNumeric. (Not yet implemented).
+ *
+ * Since: 5.0.2
+ */
+void
+gda_numeric_set_width (GdaNumeric *numeric, glong width)
+{
+	g_return_if_fail (numeric);
+	numeric->width = width;
+}
+
+/**
+ * gda_numeric_get_width:
+ * @numeric: a #GdaNumeric
+ *
+ * Gets the width of a #GdaNumeric. (Not yet implemented).
+ *
+ * Returns: an integer with the width of a #GdaNumeric. (Not jet implemented).
+ * Since: 5.0.2
+ */
+glong
+gda_numeric_get_width (GdaNumeric *numeric)
+{
+	g_return_val_if_fail (numeric, 0.0);
+	return numeric->width;
+}
+
+/**
+ * gda_numeric_set_precision:
+ * @numeric: a #GdaNumeric
+ * @precision: a #glong
+ *
+ * Sets the precision of a #GdaNumeric.
+ *
+ * Since: 5.0.2
+ */
+void
+gda_numeric_set_precision (GdaNumeric *numeric, glong precision)
+{
+	g_return_if_fail (numeric);
+	numeric->precision = precision;
+}
+
+/**
+ * gda_numeric_get_precision:
+ * @numeric: a #GdaNumeric
+ *
+ * Gets the precision of a #GdaNumeric.
+ *
+ * Returns: an integer with the precision of a #GdaNumeric.
+ * Since: 5.0.2
+ */
+glong
+gda_numeric_get_precision (GdaNumeric *numeric)
+{
+	g_return_val_if_fail (numeric, -1);
+	return numeric->precision;
+}
+/**
+ * gda_numeric_get_string:
+ * @numeric: a #GdaNumeric
+ *
+ * Get the string representation of @numeric.
+ *
+ * Returns: (transfer full) (allow-none): a new string representing the stored valued in @numeric
+ * Since: 5.0.2
+ */
+gchar*
+gda_numeric_get_string (GdaNumeric *numeric)
+{
+	g_return_val_if_fail (numeric, NULL);
+	return g_strdup (numeric->number);
+}
 
 /*
  * Register the GdaTime type in the GType system
@@ -1132,6 +1408,25 @@ gda_value_new_null (void)
 	return gda_value_new (GDA_TYPE_NULL);
 }
 
+/**
+ * gda_value_new_default:
+ * @default_val: (allow-none): the default value as a string, or %NULL
+ *
+ * Creates a new default value.
+ *
+ * Returns: (transfer full): a new #GValue of the type #GDA_TYPE_DEFAULT
+ *
+ * Since: 4.2.9
+ */
+GValue *
+gda_value_new_default (const gchar *default_val)
+{
+	GValue *value;
+	value = gda_value_new (GDA_TYPE_DEFAULT);
+	g_value_set_boxed (value, default_val);
+	return value;
+}
+
 /**
  * gda_value_new_binary:
  * @val: value to set for the new #GValue.
@@ -1330,7 +1625,7 @@ gda_value_new_from_xml (const xmlNodePtr node)
 		return NULL;
 
 	value = g_new0 (GValue, 1);
-	prop = xmlGetProp (node, (xmlChar*)"gdatype");
+	prop = xmlGetProp (node, (xmlChar*) "gdatype");
 	if (prop && !gda_value_set_from_string (value,
 						(gchar*)xmlNodeGetContent (node),
 						gda_g_type_from_string ((gchar*) prop))) {
@@ -1598,7 +1893,7 @@ gda_value_set_geometric_point (GValue *value, const GdaGeometricPoint *val)
 }
 
 /**
- * gda_value_set_null
+ * gda_value_set_null:
  * @value: a #GValue that will store a value of type #GDA_TYPE_NULL.
  *
  * Sets the type of @value to #GDA_TYPE_NULL.
@@ -1611,7 +1906,7 @@ gda_value_set_null (GValue *value)
 }
 
 /**
- * gda_value_get_numeric
+ * gda_value_get_numeric:
  * @value: a #GValue whose value we want to get.
  *
  * Returns: (transfer none): the value stored in @value.
@@ -1891,6 +2186,11 @@ gda_value_stringify (const GValue *value)
 			else
 				return g_strdup ("0000-00-00");
 		}
+		else if (G_TYPE_IS_OBJECT (type)) {
+			GObject *obj;
+			obj = g_value_get_object (value);
+			return g_strdup_printf ("%p (%s)", obj, G_OBJECT_TYPE_NAME (obj));
+		}
 		else
 			return g_strdup ("");
 	}
@@ -2288,8 +2588,13 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	}
 
 	else if (type == G_TYPE_CHAR) {
+#if GLIB_CHECK_VERSION(2,31,7)
+		gint8 c1 = g_value_get_schar (value1);
+		gint8 c2 = g_value_get_schar (value2);
+#else
 		gchar c1 = g_value_get_char (value1);
 		gchar c2 = g_value_get_char (value2);
+#endif
 		return (c1 > c2) ? 1 : ((c1 == c2) ? 0 : -1);
 	}
 
@@ -2499,7 +2804,8 @@ gda_ushort_get_type (void) {
  *
  * Converts all the non printable characters of bin->data into the "\xyz" representation
  * where "xyz" is the octal representation of the byte, and the '\' (backslash) character
- * is converted to "\\".
+ * is converted to "\\". Printable characters (defined by g_ascii_isprint()) as well as newline
+ * character are not converted.
  *
  * Note that the backslash and newline characters are considered as printable characters and
  * will not be represented by the "\xyz" representation.
@@ -2577,22 +2883,23 @@ gda_binary_to_string (const GdaBinary *bin, guint maxlen)
 
 /**
  * gda_string_to_binary:
- * @str: a string to convert
+ * @str: (allow-none): a string to convert, or %NULL
  *
  * Performs the reverse of gda_binary_to_string() (note that for any "\xyz" succession
  * of 4 characters where "xyz" represents a valid octal value, the resulting read value will
- * be modulo 256)
+ * be modulo 256).
  *
- * Returns: (transfer full): a new #GdaBinary if no error were found in @str, or NULL otherwise
+ * I @str is %NULL, then an empty (i.e. where the @data part is %NULL) #GdaBinary is created and returned.
+ *
+ * Returns: (transfer full): a new #GdaBinary if no error were found in @str, or %NULL otherwise
  */
 GdaBinary *
 gda_string_to_binary (const gchar *str)
 {
 	GdaBinary *bin;
 	glong len = 0, total;
-	gchar *rptr;
-	const gchar *sptr;
-	gchar *retval;
+	const guchar *sptr;
+	guchar *rptr, *retval;
 
 	if (!str) {
 		bin = g_new0 (GdaBinary, 1);
@@ -2603,8 +2910,8 @@ gda_string_to_binary (const gchar *str)
 
 	total = strlen (str);
 	retval = g_new0 (gchar, total + 1);
-	sptr = str;
-	rptr = (gchar *) retval;
+	sptr = (guchar*) str;
+	rptr = retval;
 
 	while (*sptr) {
 		if (*sptr == '\\') {
@@ -2613,13 +2920,15 @@ gda_string_to_binary (const gchar *str)
 				sptr += 2;
 			}
 			else {
+				guint tmp;
 				if ((*(sptr+1) >= '0') && (*(sptr+1) <= '7') &&
 				    (*(sptr+2) >= '0') && (*(sptr+2) <= '7') &&
 				    (*(sptr+3) >= '0') && (*(sptr+3) <= '7')) {
-					*rptr = (*(sptr+1) - '0') * 64 +
+					tmp = (*(sptr+1) - '0') * 64 +
 						(*(sptr+2) - '0') * 8 +
 						(*(sptr+3) - '0');
 					sptr += 4;
+					*rptr = tmp % 256;
 				}
 				else {
 					g_free (retval);
@@ -2637,7 +2946,7 @@ gda_string_to_binary (const gchar *str)
 	}
 
 	bin = g_new0 (GdaBinary, 1);
-	bin->data = (guchar*)retval;
+	bin->data = retval;
 	bin->binary_length = len;
 
 	return bin;
