@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 - 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,10 +38,10 @@ typedef struct _GdaPStmtClass   GdaPStmtClass;
 /**
  * GdaPStmt:
  * @sql: actual SQL code used for this prepared statement, mem freed by GdaPStmt
- * @param_ids: (element-type string): list of parameters' IDs (as gchar *), mem freed by GdaPStmt
+ * @param_ids: (element-type utf8): list of parameters' IDs (as gchar *), mem freed by GdaPStmt
  * @ncols: number of types in array
  * @types: (array length=ncols) (element-type GLib.Type): array of ncols types
- * @tmpl_columns: (element-type Gda.Colum): list of #GdaColumn objects which data models created from this prep. statement can copy
+ * @tmpl_columns: (element-type Gda.Column): list of #GdaColumn objects which data models created from this prep. statement can copy
  *
  */
 struct _GdaPStmt {
@@ -66,6 +66,7 @@ struct _GdaPStmt {
 struct _GdaPStmtClass {
 	GObjectClass  parent_class;
 
+	/*< private >*/
 	/* Padding for future expansion */
 	void (*_gda_reserved1) (void);
 	void (*_gda_reserved2) (void);
@@ -73,6 +74,34 @@ struct _GdaPStmtClass {
 	void (*_gda_reserved4) (void);
 };
 
+/**
+ * SECTION:gda-pstmt
+ * @short_description: Base class for prepared statement's
+ * @title: GdaPstmt
+ * @stability: Stable
+ * @see_also:
+ *
+ * The #GdaPStmt represents the association between a #GdaStatement statement and a <emphasis>prepared statement</emphasis>
+ * which is database dependent and is an in-memory representation of a statement. Using prepared statement has the
+ * following advantages:
+ * <itemizedlist>
+ *   <listitem><para>the parsing of the SQL has to be done only once, which improves performances if the statement
+ *	has to be executed more than once</para></listitem>
+ *   <listitem><para>if a statement has been prepared, then it means it is syntactically correct and has been
+ *	<emphasis>understood</emphasis> by the database's API</para></listitem>
+ *   <listitem><para>it is possible to use variables in prepared statement which eliminates the risk
+ *	of SQL code injection</para></listitem>
+ * </itemizedlist>
+ *
+ * The #GdaPStmt is not intended to be instantiated, but subclassed by database provider's implementation.
+ * Once created, the database provider's implementation can decide to associate (for future lookup) to
+ * a #GdaStatement object in a connection using gda_connection_add_prepared_statement().
+ *
+ * The #GdaPStmt object can keep a reference to the #GdaStatement object (which can be set and get using
+ * the gda_pstmt_set_gda_statement() and gda_pstmt_get_gda_statement()), however that reference
+ * if a weak one (which means it will be lost if the #GdaStatement object is destroyed).
+ */
+
 GType         gda_pstmt_get_type          (void) G_GNUC_CONST;
 void          gda_pstmt_set_gda_statement (GdaPStmt *pstmt, GdaStatement *stmt);
 void          gda_pstmt_copy_contents     (GdaPStmt *src, GdaPStmt *dest);
