@@ -241,8 +241,11 @@ _gda_sqlite_recordset_new (GdaConnection *cnc, GdaSqlitePStmt *ps, GdaSet *exec_
 									 gda_column_new ());
 		_GDA_PSTMT (ps)->tmpl_columns = g_slist_reverse (_GDA_PSTMT (ps)->tmpl_columns);
 
-		/* create prepared statement's types */
-		_GDA_PSTMT (ps)->types = g_new0 (GType, _GDA_PSTMT (ps)->ncols); /* all types are initialized to GDA_TYPE_NULL */
+		/* create prepared statement's types, all types are initialized to GDA_TYPE_NULL */
+		_GDA_PSTMT (ps)->types = g_new (GType, _GDA_PSTMT (ps)->ncols);
+		for (i = 0; i < _GDA_PSTMT (ps)->ncols; i++)
+			_GDA_PSTMT (ps)->types [i] = GDA_TYPE_NULL;
+
 		if (col_types) {
 			for (i = 0; ; i++) {
 				if (col_types [i] > 0) {
@@ -346,8 +349,8 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 	GdaSqlitePStmt *ps;
 	GdaRow *prow = NULL;
 
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data 
-		(gda_data_select_get_connection ((GdaDataSelect*) model));
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error 
+		(gda_data_select_get_connection ((GdaDataSelect*) model), error);
 	if (!cdata)
 		return NULL;
 	ps = GDA_SQLITE_PSTMT (GDA_DATA_SELECT (model)->prep_stmt);
@@ -416,8 +419,7 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 				gda_value_set_null (value);
 			}
 			else {
-				if (type != GDA_TYPE_NULL)
-					g_value_init (value, type);
+				gda_value_reset_with_type (value, type);
 				
 				if (type == GDA_TYPE_NULL)
 					;
@@ -576,7 +578,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
+#if GLIB_CHECK_VERSION(2,31,7)
+						g_value_set_schar (value, (gchar) i);
+#else
 						g_value_set_char (value, (gchar) i);
+#endif
 				}
 				else if (type == G_TYPE_UCHAR) {
 					gint64 i;
@@ -654,7 +660,7 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 		SQLITE3_CALL (sqlite3_reset) (ps->sqlite_stmt);
 		if (rc == SQLITE_IOERR_TRUNCATE)
 			g_set_error (&lerror, GDA_DATA_MODEL_ERROR,
-				     GDA_DATA_MODEL_TRUNCATED_ERROR, _("Truncated data"));
+				     GDA_DATA_MODEL_TRUNCATED_ERROR, "%s", _("Truncated data"));
 		else
 			g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_INTERNAL_ERROR, 
