@@ -770,7 +770,7 @@ gda_ldap_connection_undeclare_table (GdaLdapConnection *cnc, const gchar *table_
 	if (specs && ! g_slist_find (cnc->priv->maps, specs)) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_MISUSE_ERROR,
-			     _("Can't remove non LDAP virtual table"));
+			     "%s", _("Can't remove non LDAP virtual table"));
 		return FALSE;
 	}
 	return gda_vconnection_data_model_remove (GDA_VCONNECTION_DATA_MODEL (cnc), table_name, error);
@@ -818,14 +818,14 @@ gda_ldap_connection_describe_table (GdaLdapConnection *cnc, const gchar *table_n
 	if (specs && ! g_slist_find (cnc->priv->maps, specs)) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_MISUSE_ERROR,
-			     _("Can't describe non LDAP virtual table"));
+			     "%s", _("Can't describe non LDAP virtual table"));
 		return FALSE;
 	}
 	
 	if (!specs) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 			     GDA_SERVER_PROVIDER_MISUSE_ERROR,
-			     _("Unknown LDAP virtual table"));
+			     "%s", _("Unknown LDAP virtual table"));
 		return FALSE;
 	}
 
@@ -1023,3 +1023,48 @@ gda_ldap_get_top_classes (GdaLdapConnection *cnc)
 
 	return _gda_ldap_get_top_classes (cnc);
 }
+
+GSList *_gda_ldap_entry_get_attributes_list (GdaLdapConnection *cnc, GdaLdapEntry *entry, GdaLdapAttribute *object_class_attr);
+/**
+ * gda_ldap_entry_get_attributes_list:
+ * @entry: a #GdaLdapEntry
+ *
+ * Get a list of all the possible attributes which @entry can have. Each possible attribute is represented
+ * by a #GdaLdapAttributeDefinition strunture.
+ *
+ * Returns: (transfer full) (element-type GdaLdapAttributeDefinition): a #GSList of #GdaLdapAttributeDefinition pointers, free the list using gda_ldap_attributes_list_free()
+ *
+ * Since: 5.2.0
+ */
+GSList *
+gda_ldap_entry_get_attributes_list (GdaLdapConnection *cnc, GdaLdapEntry *entry)
+{
+	g_return_val_if_fail (entry, NULL);
+
+	return _gda_ldap_entry_get_attributes_list (cnc, entry, NULL);
+}
+
+/**
+ * gda_ldap_attributes_list_free:
+ * @list: (allow-none): a #GSList of #GdaLdapAttributeDefinition pointers, or %NULL
+ *
+ * Frees the list returned by gda_ldap_entry_get_attributes_list().
+ *
+ * Since: 5.2.0
+ */
+void
+gda_ldap_attributes_list_free (GSList *list)
+{
+	GSList *l;
+	if (!list)
+		return;
+	for (l = list; l; l = l->next) {
+		GdaLdapAttributeDefinition *def;
+		def = (GdaLdapAttributeDefinition*) l->data;
+		if (def) {
+			g_free (def->name);
+			g_free (def);
+		}
+	}
+	g_slist_free (list);
+}
