@@ -40,7 +40,11 @@
  * by the worker thread. It is used to avoid creating signal data for threads for which
  * no job is being performed 
  */
+#if GLIB_CHECK_VERSION(2,31,7)
+GPrivate worker_thread_current_queue;
+#else
 GStaticPrivate worker_thread_current_queue = G_STATIC_PRIVATE_INIT;
+#endif
 
 typedef struct _ThreadData ThreadData;
 typedef struct _Job Job;
@@ -48,7 +52,11 @@ typedef struct _SignalSpec SignalSpec;
 typedef struct _Pipe Pipe;
 
 struct _GdaThreadWrapperPrivate {
+#if GLIB_CHECK_VERSION(2,31,7)
+	GRecMutex    rmutex;
+#else
 	GdaMutex    *mutex;
+#endif
 	guint        next_job_id;
 	GThread     *worker_thread;
 	GAsyncQueue *to_worker_thread;
@@ -68,12 +76,21 @@ struct _Pipe {
 	int          fds[2]; /* [0] for reading and [1] for writing */
 	GIOChannel  *ioc;
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	GMutex       mutex;
+#else
 	GMutex      *mutex; /* locks @ref_count */
+#endif
 	guint        ref_count;
 };
 
+#if GLIB_CHECK_VERSION(2,31,7)
+#define pipe_lock(x) g_mutex_lock(& (((Pipe*)x)->mutex))
+#define pipe_unlock(x) g_mutex_unlock(& (((Pipe*)x)->mutex))
+#else
 #define pipe_lock(x) g_mutex_lock(((Pipe*)x)->mutex);
 #define pipe_unlock(x) g_mutex_unlock(((Pipe*)x)->mutex);
+#endif
 
 static Pipe *
 pipe_ref (Pipe *p)
@@ -98,9 +115,14 @@ pipe_unref (Pipe *p)
 #ifdef DEBUG_NOTIFICATION
 		g_print ("Pipe %p --: %u\n", p, p->ref_count);
 #endif
-		pipe_unlock (p);
 		if (p->ref_count == 0) {
 			/* destroy @p */
+#if GLIB_CHECK_VERSION(2,31,7)
+			GMutex *m = &(p->mutex);
+#else
+			GMutex *m = p->mutex;
+#endif
+
 			if (p->ioc)
 				g_io_channel_unref (p->ioc);
 #ifdef G_OS_WIN32
@@ -114,12 +136,22 @@ pipe_unref (Pipe *p)
 			if (p->fds[1] >= 0)
 				close (p->fds[1]);
 #endif
-			g_mutex_free (p->mutex);
+
 #ifdef DEBUG_NOTIFICATION
 			g_print ("Destroyed Pipe %p\n", p);
 #endif
 			g_free (p);
+
+			g_mutex_unlock (m);
+#if GLIB_CHECK_VERSION(2,31,7)
+			g_mutex_clear (m);
+#else
+			g_mutex_free (m);
+#endif
+
 		}
+		else
+			pipe_unlock (p);
 	}
 }
 
@@ -132,7 +164,11 @@ pipe_new (void)
 	Pipe *p;
 
 	p = g_new0 (Pipe, 1);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_mutex_init (&(p->mutex));
+#else
 	p->mutex = g_mutex_new ();
+#endif
 	p->ref_count = 1;
 	p->thread = g_thread_self ();
 #ifdef G_OS_WIN32
@@ -167,10 +203,20 @@ pipe_new (void)
 static Pipe *
 get_pipe (GdaThreadWrapper *wrapper, GThread *thread)
 {
+	Pipe *p = NULL;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
+	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	if (wrapper->priv->pipes_hash)
-		return g_hash_table_lookup (wrapper->priv->pipes_hash, thread);
-	else
-		return NULL;
+		p = g_hash_table_lookup (wrapper->priv->pipes_hash, thread);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
+	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
+	return p;
 }
 
 /* 
@@ -266,12 +312,21 @@ struct _SignalSpec {
         GdaThreadWrapperCallback callback;
         gpointer                 data;
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	GMutex        mutex;
+#else
 	GMutex       *mutex;
+#endif
 	guint         ref_count;
 };
 
+#if GLIB_CHECK_VERSION(2,31,7)
+#define signal_spec_lock(x) g_mutex_lock(& (((SignalSpec*)x)->mutex))
+#define signal_spec_unlock(x) g_mutex_unlock(& (((SignalSpec*)x)->mutex))
+#else
 #define signal_spec_lock(x) g_mutex_lock(((SignalSpec*)x)->mutex);
 #define signal_spec_unlock(x) g_mutex_unlock(((SignalSpec*)x)->mutex);
+#endif
 
 /*
  * call signal_spec_lock() before calling this function
@@ -282,7 +337,11 @@ signal_spec_unref (SignalSpec *sigspec)
 	sigspec->ref_count --;
 	if (sigspec->ref_count == 0) {
 		signal_spec_unlock (sigspec);
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_mutex_clear (&(sigspec->mutex));
+#else
 		g_mutex_free (sigspec->mutex);
+#endif
 		if (sigspec->instance && (sigspec->signal_id > 0))
 			g_signal_handler_disconnect (sigspec->instance, sigspec->signal_id);
 		if (sigspec->reply_queue)
@@ -325,8 +384,11 @@ static ThreadData *
 get_thread_data (GdaThreadWrapper *wrapper, GThread *thread)
 {
 	ThreadData *td;
-
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, thread);
 	if (!td) {
 		Pipe *p;
@@ -340,7 +402,11 @@ get_thread_data (GdaThreadWrapper *wrapper, GThread *thread)
 		td->notif = pipe_ref (p);
 		g_hash_table_insert (wrapper->priv->threads_hash, thread, td);
 	}
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	return td;
 }
 
@@ -443,6 +509,8 @@ clean_notifications (GdaThreadWrapper *wrapper, ThreadData *td)
  * @td: (allow-none): may be %NULL
  *
  * Either @wrapper and @td are both NULL, or they are both NOT NULL
+ *
+ * It is assumed that pipe_ref(p) has been called before calling this function
  */
 static gboolean
 write_notification (GdaThreadWrapper *wrapper, ThreadData *td,
@@ -450,8 +518,10 @@ write_notification (GdaThreadWrapper *wrapper, ThreadData *td,
 {
 	g_assert ((wrapper && td) || (!wrapper && !td));
 
-	if (!p || (p->fds[1] < 0))
+	if (!p || (p->fds[1] < 0)) {
+		pipe_unref (p);
 		return TRUE;
+	}
 #ifdef DEBUG_NOTIFICATION_FORCE
 	static guint c = 0;
 	c++;
@@ -475,6 +545,7 @@ write_notification (GdaThreadWrapper *wrapper, ThreadData *td,
 #ifdef DEBUG_NOTIFICATION
 	g_print ("Wrote notification %d.%u to pipe %p\n", type, job_id, p);
 #endif
+	pipe_unref (p);
 	return TRUE;
 
  onerror:
@@ -489,6 +560,7 @@ write_notification (GdaThreadWrapper *wrapper, ThreadData *td,
 	close (p->fds [1]);
 #endif
 	p->fds [1] = -1;
+	pipe_unref (p);
 	if (td)
 		clean_notifications (wrapper, td);
 	return FALSE;
@@ -510,11 +582,19 @@ worker_thread_entry_point (GAsyncQueue *to_worker_thread)
 		Job *job;
 		
 		/* pop next job and mark it as being processed */
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_private_set (&worker_thread_current_queue, NULL);
+#else
 		g_static_private_set (&worker_thread_current_queue, NULL, NULL);
+#endif
 		g_async_queue_lock (in);
 		job = g_async_queue_pop_unlocked (in);
 		job->processed = TRUE;
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_private_set (&worker_thread_current_queue, job->reply_queue);
+#else
 		g_static_private_set (&worker_thread_current_queue, job->reply_queue, NULL);
+#endif
 		g_async_queue_unlock (in);
 
 		if (job->cancelled) {
@@ -539,9 +619,11 @@ worker_thread_entry_point (GAsyncQueue *to_worker_thread)
 				job->u.exe.result = NULL;
 				job->void_func (job->arg, &(job->u.exe.error));
 			}
+
+			guint jid = job->job_id;
+			Pipe *jpipe = pipe_ref (job->notif);
 			g_async_queue_push (job->reply_queue, job);
-			if (! write_notification (NULL, NULL, job->notif, GDA_THREAD_NOTIFICATION_JOB,
-						 job->job_id)) {
+			if (! write_notification (NULL, NULL, jpipe, GDA_THREAD_NOTIFICATION_JOB, jid)) {
 				Job *je = g_new0 (Job, 1);
 				je->type = JOB_TYPE_NOTIFICATION_ERROR;
 				g_async_queue_push (job->reply_queue, je);
@@ -562,15 +644,25 @@ gda_thread_wrapper_init (GdaThreadWrapper *wrapper, G_GNUC_UNUSED GdaThreadWrapp
 	g_return_if_fail (GDA_IS_THREAD_WRAPPER (wrapper));
 
 	wrapper->priv = g_new0 (GdaThreadWrapperPrivate, 1);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_init (&(wrapper->priv->rmutex));
+#else
 	wrapper->priv->mutex = gda_mutex_new ();
+#endif
 	wrapper->priv->next_job_id = 1;
 
 	wrapper->priv->threads_hash = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify) thread_data_free);
 
 	wrapper->priv->to_worker_thread = g_async_queue_new ();
+#if GLIB_CHECK_VERSION(2,31,7)
+	wrapper->priv->worker_thread = g_thread_new ("worker",
+						     (GThreadFunc) worker_thread_entry_point,
+						     g_async_queue_ref (wrapper->priv->to_worker_thread)); /* inc. ref for sub thread usage */
+#else
 	wrapper->priv->worker_thread = g_thread_create ((GThreadFunc) worker_thread_entry_point,
-						     g_async_queue_ref (wrapper->priv->to_worker_thread), /* inc. ref for sub thread usage */
-						     FALSE, NULL);
+							g_async_queue_ref (wrapper->priv->to_worker_thread), /* inc. ref for sub thread usage */
+							FALSE, NULL);
+#endif
 	
 	wrapper->priv->pipes_hash = NULL;
 
@@ -636,7 +728,11 @@ gda_thread_wrapper_dispose (GObject *object)
 		g_async_queue_unref (wrapper->priv->to_worker_thread);
 		wrapper->priv->worker_thread = NULL; /* side note: don't wait for sub thread to terminate */
 
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_clear (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_free (wrapper->priv->mutex);
+#endif
 
 		if (wrapper->priv->pipes_hash)
 			g_hash_table_destroy (wrapper->priv->pipes_hash);
@@ -659,7 +755,7 @@ GQuark gda_thread_wrapper_error_quark (void)
 }
 
 /**
- * gda_thread_wrapper_get_type
+ * gda_thread_wrapper_get_type:
  * 
  * Registers the #GdaThreadWrapper class on the GLib type system.
  * 
@@ -730,7 +826,7 @@ gda_thread_wrapper_get_property (GObject *object,
 }
 
 /**
- * gda_thread_wrapper_new
+ * gda_thread_wrapper_new:
  *
  * Creates a new #GdaThreadWrapper object
  *
@@ -783,7 +879,11 @@ gda_thread_wrapper_get_io_channel (GdaThreadWrapper *wrapper)
 	g_return_val_if_fail (wrapper->priv, NULL);
 
 	th = g_thread_self ();
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	p = get_pipe (wrapper, th);
 	if (!p) {
 		p = pipe_new ();
@@ -796,7 +896,11 @@ gda_thread_wrapper_get_io_channel (GdaThreadWrapper *wrapper)
 			g_hash_table_insert (wrapper->priv->pipes_hash, th, p);
 		}
 	}
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	if (p)
 		return p->ioc;
 	else
@@ -820,7 +924,11 @@ gda_thread_wrapper_unset_io_channel (GdaThreadWrapper *wrapper)
 	g_return_if_fail (GDA_IS_THREAD_WRAPPER (wrapper));
 	g_return_if_fail (wrapper->priv);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	th = g_thread_self ();
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, th);
 	if (td) {
@@ -829,7 +937,11 @@ gda_thread_wrapper_unset_io_channel (GdaThreadWrapper *wrapper)
 		if (p)
 			clean_notifications (wrapper, td);
 	}
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 }
 
 /**
@@ -873,9 +985,17 @@ gda_thread_wrapper_execute (GdaThreadWrapper *wrapper, GdaThreadWrapperFunc func
 	job->type = JOB_TYPE_EXECUTE;
 	job->processed = FALSE;
 	job->cancelled = FALSE;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	job->job_id = wrapper->priv->next_job_id++;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	job->func = func;
 	job->void_func = NULL;
 	job->arg = arg;
@@ -901,8 +1021,10 @@ gda_thread_wrapper_execute (GdaThreadWrapper *wrapper, GdaThreadWrapperFunc func
 #ifdef THREAD_WRAPPER_DEBUG
 		g_print ("... IMMEDIATELY done job %d => %p\n", job->job_id, job->u.exe.result);
 #endif
+		guint jid = job->job_id;
+		Pipe *jpipe = pipe_ref (job->notif);
                 g_async_queue_push (job->reply_queue, job);
-		write_notification (wrapper, td, job->notif, GDA_THREAD_NOTIFICATION_JOB, job->job_id);
+		write_notification (wrapper, td, jpipe, GDA_THREAD_NOTIFICATION_JOB, jid);
         }
         else
                 g_async_queue_push (wrapper->priv->to_worker_thread, job);
@@ -953,9 +1075,17 @@ gda_thread_wrapper_execute_void (GdaThreadWrapper *wrapper, GdaThreadWrapperVoid
 	job->type = JOB_TYPE_EXECUTE;
 	job->processed = FALSE;
 	job->cancelled = FALSE;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	job->job_id = wrapper->priv->next_job_id++;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	job->func = NULL;
 	job->void_func = func;
 	job->arg = arg;
@@ -981,8 +1111,10 @@ gda_thread_wrapper_execute_void (GdaThreadWrapper *wrapper, GdaThreadWrapperVoid
 #ifdef THREAD_WRAPPER_DEBUG
 		g_print ("... IMMEDIATELY done VOID job %d => %p\n", job->job_id, job->u.exe.result);
 #endif
+		guint jid = job->job_id;
+		Pipe *jpipe = pipe_ref (job->notif);
                 g_async_queue_push (job->reply_queue, job);
-		write_notification (wrapper, td, job->notif, GDA_THREAD_NOTIFICATION_JOB, job->job_id);
+		write_notification (wrapper, td, jpipe, GDA_THREAD_NOTIFICATION_JOB, jid);
         }
         else
 		g_async_queue_push (wrapper->priv->to_worker_thread, job);
@@ -991,7 +1123,7 @@ gda_thread_wrapper_execute_void (GdaThreadWrapper *wrapper, GdaThreadWrapperVoid
 }
 
 /**
- * gda_thread_wrapper_cancel
+ * gda_thread_wrapper_cancel:
  * @wrapper: a #GdaThreadWrapper object
  * @id: the ID of a job as returned by gda_thread_wrapper_execute() or gda_thread_wrapper_execute_void()
  * 
@@ -1014,13 +1146,21 @@ gda_thread_wrapper_cancel (GdaThreadWrapper *wrapper, guint id)
 	ThreadData *td;
 
 	g_return_val_if_fail (GDA_IS_THREAD_WRAPPER (wrapper), FALSE);
-	
+
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, g_thread_self());
 	if (!td) {
 		/* nothing to be done for this thread */
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return FALSE;
 	}
 
@@ -1044,13 +1184,17 @@ gda_thread_wrapper_cancel (GdaThreadWrapper *wrapper, guint id)
 		}
 	}
 	g_async_queue_unlock (wrapper->priv->to_worker_thread);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 
 	return retval;
 }
 
 /**
- * gda_thread_wrapper_iterate
+ * gda_thread_wrapper_iterate:
  * @wrapper: a #GdaThreadWrapper object
  * @may_block: whether the call may block
  *
@@ -1076,9 +1220,17 @@ gda_thread_wrapper_iterate (GdaThreadWrapper *wrapper, gboolean may_block)
 	g_return_if_fail (GDA_IS_THREAD_WRAPPER (wrapper));
 	g_return_if_fail (wrapper->priv);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, g_thread_self());
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	if (!td) {
 		/* nothing to be done for this thread */
 		return;
@@ -1160,9 +1312,17 @@ gda_thread_wrapper_fetch_result (GdaThreadWrapper *wrapper, gboolean may_lock, g
 	g_return_val_if_fail (wrapper->priv, NULL);
 	g_return_val_if_fail (exp_id > 0, NULL);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, g_thread_self());
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	if (!td) {
 		/* nothing to be done for this thread */
 		return NULL;
@@ -1182,9 +1342,17 @@ gda_thread_wrapper_fetch_result (GdaThreadWrapper *wrapper, gboolean may_lock, g
 					    (g_async_queue_length (td->from_worker_thread) == 0) &&
 					    !td->signals_list) {
 						/* remove this ThreadData */
+#if GLIB_CHECK_VERSION(2,31,7)
+						g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 						gda_mutex_lock (wrapper->priv->mutex);
+#endif
 						g_hash_table_remove (wrapper->priv->threads_hash, g_thread_self());
+#if GLIB_CHECK_VERSION(2,31,7)
+						g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 						gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 					}
 					goto out;
 				}
@@ -1220,7 +1388,7 @@ gda_thread_wrapper_fetch_result (GdaThreadWrapper *wrapper, gboolean may_lock, g
 }
 
 /**
- * gda_thread_wrapper_get_waiting_size
+ * gda_thread_wrapper_get_waiting_size:
  * @wrapper: a #GdaThreadWrapper object
  *
  * Use this method to query the number of functions which have been queued to be executed
@@ -1240,11 +1408,19 @@ gda_thread_wrapper_get_waiting_size (GdaThreadWrapper *wrapper)
 	g_return_val_if_fail (GDA_IS_THREAD_WRAPPER (wrapper), 0);
 	g_return_val_if_fail (wrapper->priv, 0);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, g_thread_self());
 	if (!td) {
 		/* nothing to be done for this thread */
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return 0;
 	}
 	
@@ -1255,7 +1431,11 @@ gda_thread_wrapper_get_waiting_size (GdaThreadWrapper *wrapper)
 			size++;
 	}
 	g_async_queue_unlock (wrapper->priv->to_worker_thread);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 	return size;
 }
 
@@ -1278,9 +1458,15 @@ worker_thread_closure_marshal (GClosure *closure,
 		return;
 
 	/* check that the worker thread is working on a job for which job->reply_queue == sigspec->reply_queue */
+#if GLIB_CHECK_VERSION(2,31,7)
+	if (sigspec->private &&
+	    g_private_get (&worker_thread_current_queue) != sigspec->reply_queue)
+		return;
+#else
 	if (sigspec->private &&
 	    g_static_private_get (&worker_thread_current_queue) != sigspec->reply_queue)
 		return;
+#endif
 
 	gsize i;
 	/*
@@ -1300,14 +1486,13 @@ worker_thread_closure_marshal (GClosure *closure,
 		src = param_values + i;
 		dest = job->u.signal.param_values + i - 1;
 
-		if (G_VALUE_TYPE (src) != GDA_TYPE_NULL) {
-			g_value_init (dest, G_VALUE_TYPE (src));
-			g_value_copy (src, dest);
-		}
+		g_value_init (dest, G_VALUE_TYPE (src));
+		g_value_copy (src, dest);
 	}
 
+	Pipe *jpipe = pipe_ref (sigspec->notif);
 	g_async_queue_push (sigspec->reply_queue, job);
-	if (! write_notification (NULL, NULL, sigspec->notif, GDA_THREAD_NOTIFICATION_SIGNAL, 0)) {
+	if (! write_notification (NULL, NULL, jpipe, GDA_THREAD_NOTIFICATION_SIGNAL, 0)) {
 		Job *je = g_new0 (Job, 1);
 		je->type = JOB_TYPE_NOTIFICATION_ERROR;
 		g_async_queue_push (sigspec->reply_queue, je);
@@ -1348,14 +1533,13 @@ worker_thread_closure_marshal_anythread (GClosure *closure,
 		src = param_values + i;
 		dest = job->u.signal.param_values + i - 1;
 
-		if (G_VALUE_TYPE (src) != GDA_TYPE_NULL) {
-			g_value_init (dest, G_VALUE_TYPE (src));
-			g_value_copy (src, dest);
-		}
+		g_value_init (dest, G_VALUE_TYPE (src));
+		g_value_copy (src, dest);
 	}
 
+	Pipe *jpipe = pipe_ref (sigspec->notif);
 	g_async_queue_push (sigspec->reply_queue, job);
-	if (! write_notification (NULL, NULL, sigspec->notif, GDA_THREAD_NOTIFICATION_SIGNAL, 0)) {
+	if (! write_notification (NULL, NULL, jpipe, GDA_THREAD_NOTIFICATION_SIGNAL, 0)) {
 		Job *je = g_new0 (Job, 1);
 		je->type = JOB_TYPE_NOTIFICATION_ERROR;
 		g_async_queue_push (sigspec->reply_queue, je);
@@ -1418,7 +1602,11 @@ gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
 	g_return_val_if_fail (GDA_IS_THREAD_WRAPPER (wrapper), 0);
 	g_return_val_if_fail (wrapper->priv, 0);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 	
 	td = get_thread_data (wrapper, g_thread_self());
 
@@ -1444,7 +1632,11 @@ gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
         sigspec->instance = instance;
         sigspec->callback = callback;
         sigspec->data = data;
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_mutex_init (&(sigspec->mutex));
+#else
 	sigspec->mutex = g_mutex_new ();
+#endif
 	sigspec->ref_count = 1;
 
 	GClosure *cl;
@@ -1457,7 +1649,11 @@ gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
 
 	td->signals_list = g_slist_append (td->signals_list, sigspec);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 
 	return sigspec->signal_id;
 }
@@ -1476,7 +1672,7 @@ find_signal_r_func (G_GNUC_UNUSED GThread *thread, ThreadData *td, gulong *id)
 }
 
 /**
- * gda_thread_wrapper_disconnect
+ * gda_thread_wrapper_disconnect:
  * @wrapper: a #GdaThreadWrapper object
  * @id: a handler ID, as returned by gda_thread_wrapper_connect_raw()
  *
@@ -1498,7 +1694,11 @@ gda_thread_wrapper_disconnect (GdaThreadWrapper *wrapper, gulong id)
 	g_return_if_fail (GDA_IS_THREAD_WRAPPER (wrapper));
 	g_return_if_fail (wrapper->priv);
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 
 	td = g_hash_table_lookup (wrapper->priv->threads_hash, g_thread_self());
 	if (!td) {
@@ -1508,7 +1708,11 @@ gda_thread_wrapper_disconnect (GdaThreadWrapper *wrapper, gulong id)
 	}
 	if (!td) {
 		g_warning (_("Signal %lu does not exist"), id);
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return;
 	}
 
@@ -1521,7 +1725,11 @@ gda_thread_wrapper_disconnect (GdaThreadWrapper *wrapper, gulong id)
 
 	if (!sigspec) {
 		g_warning (_("Signal does not exist\n"));
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return;
 	}
 
@@ -1546,12 +1754,15 @@ gda_thread_wrapper_disconnect (GdaThreadWrapper *wrapper, gulong id)
 		/* remove this ThreadData */
 		g_hash_table_remove (wrapper->priv->threads_hash, g_thread_self());
 	}
-
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 }
 
 /**
- * gda_thread_wrapper_steal_signal
+ * gda_thread_wrapper_steal_signal:
  * @wrapper: a #GdaThreadWrapper object
  * @id: a signal ID
  *
@@ -1570,19 +1781,33 @@ gda_thread_wrapper_steal_signal (GdaThreadWrapper *wrapper, gulong id)
         g_return_if_fail (wrapper->priv);
 	g_return_if_fail (id > 0);
 
-        gda_mutex_lock (wrapper->priv->mutex);
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_lock (&(wrapper->priv->rmutex));
+#else
+	gda_mutex_lock (wrapper->priv->mutex);
+#endif
 
 	gulong theid = id;
 	old_td = g_hash_table_find (wrapper->priv->threads_hash,
 				    (GHRFunc) find_signal_r_func, &theid);
 	if (!old_td) {
 		g_warning (_("Signal %lu does not exist"), id);
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return;
 	}
 
-	if (old_td->owner == g_thread_self ())
+	if (old_td->owner == g_thread_self ()) {
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
+		gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 		return;
+	}
 
         /* merge old_td and new_td */
         if (old_td->signals_list) {
@@ -1600,5 +1825,9 @@ gda_thread_wrapper_steal_signal (GdaThreadWrapper *wrapper, gulong id)
                 }
         }
 
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_rec_mutex_unlock (&(wrapper->priv->rmutex));
+#else
 	gda_mutex_unlock (wrapper->priv->mutex);
+#endif
 }
