@@ -798,8 +798,7 @@ gdaprov_ldap_get_top_classes (GdaLdapConnection *cnc)
 		return NULL;
 	if (! cdata->classes_hash) {
 		/* force classes init */
-		GdaLdapClass *lcl;
-		lcl = gdaprov_ldap_get_class_info (cnc, "top");
+		gdaprov_ldap_get_class_info (cnc, "top");
 	}
 	return cdata->top_classes;
 }
@@ -816,13 +815,13 @@ gda_ldap_get_g_type (LdapConnectionData *cdata, const gchar *attribute_name, con
 	GType coltype = GDA_TYPE_NULL;
 	if (specified_gtype)
 		coltype = gda_g_type_from_string (specified_gtype);
-	if (coltype == GDA_TYPE_NULL) {
+	if ((coltype == G_TYPE_INVALID) || (coltype == GDA_TYPE_NULL)) {
 		LdapAttribute *lat;
 		lat = gda_ldap_get_attr_info (cdata, attribute_name);
 		if (lat)
 			coltype = lat->type->gtype;
 	}
-	if (coltype == GDA_TYPE_NULL)
+	if ((coltype == G_TYPE_INVALID) || (coltype == GDA_TYPE_NULL))
 		coltype = G_TYPE_STRING;
 	return coltype;
 }
@@ -1066,6 +1065,9 @@ gda_ldap_parse_dn (const char *attr, gchar **out_userdn)
 	if (out_userdn)
 		*out_userdn = NULL;
 
+	if (!attr)
+		return FALSE;
+
 	/* decoding */
 	if (ldap_str2dn (attr, &tmpDN, LDAP_DN_FORMAT_LDAPV3) != LDAP_SUCCESS) {
 		if (ldap_str2dn (attr, &tmpDN, LDAP_DN_FORMAT_LDAPV2) != LDAP_SUCCESS) {
@@ -1138,8 +1140,10 @@ gdaprov_ldap_describe_entry (GdaLdapConnection *cnc, const gchar *dn, GError **e
 			return NULL;
 		}
 		else if (nb_entries > 1) {
-			g_set_error (error, 0, 0,
-				     _("LDAP server returned more than one entry with DN '%s'"), real_dn);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_INTERNAL_ERROR,
+				     _("LDAP server returned more than one entry with DN '%s'"),
+				     real_dn);
 			return NULL;
 		}
 
@@ -1270,7 +1274,9 @@ gdaprov_ldap_get_entry_children (GdaLdapConnection *cnc, const gchar *dn, gchar
 				}
 				g_array_free (children, TRUE);
 				children = NULL;
-				g_set_error (error, 0, 0,
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_INTERNAL_ERROR,
+					     "%s",
 					     _("Could not parse distinguished name returned by LDAP server"));
 				break;
 			}
@@ -1439,3 +1445,93 @@ gdaprov_ldap_get_base_dn (GdaLdapConnection *cnc)
 	else
 		return cdata->base_dn;
 }
+
+static gint
+def_cmp_func (GdaLdapAttributeDefinition *def1, GdaLdapAttributeDefinition *def2)
+{
+	return strcmp (def1->name, def2->name);
+}
+
+static GSList *
+handle_ldap_class (LdapConnectionData *cdata, GdaLdapClass *kl, GSList *retlist, GHashTable *hash)
+{
+	guint j;
+	/*g_print ("Class [%s] (%s)\n", kl->oid, kl->names[0]);*/
+	for (j = 0; j < kl->nb_req_attributes; j++) {
+		/*g_print ("  attr [%s] REQ\n", kl->req_attributes [j]);*/
+		GdaLdapAttributeDefinition *def;
+		LdapAttribute *latt;
+		latt = gda_ldap_get_attr_info (cdata, kl->req_attributes [j]);
+		def = g_hash_table_lookup (hash, kl->req_attributes [j]);
+		if (def)
+			def->required = TRUE;
+		else {
+			def = g_new0 (GdaLdapAttributeDefinition, 1);
+			def->name = g_strdup (kl->req_attributes [j]);
+			def->required = TRUE;
+			def->g_type = latt ? latt->type->gtype : G_TYPE_STRING;
+			g_hash_table_insert (hash, def->name, def);
+			retlist = g_slist_insert_sorted (retlist, def, (GCompareFunc) def_cmp_func);
+		}
+	}
+	for (j = 0; j < kl->nb_opt_attributes; j++) {
+		/*g_print ("  attr [%s] OPT\n", kl->opt_attributes [j]);*/
+		GdaLdapAttributeDefinition *def;
+		LdapAttribute *latt;
+		latt = gda_ldap_get_attr_info (cdata, kl->opt_attributes [j]);
+		def = g_hash_table_lookup (hash, kl->opt_attributes [j]);
+		if (!def) {
+			def = g_new0 (GdaLdapAttributeDefinition, 1);
+			def->name = g_strdup (kl->opt_attributes [j]);
+			def->required = FALSE;
+			def->g_type = latt ? latt->type->gtype : G_TYPE_STRING;
+			g_hash_table_insert (hash, def->name, def);
+			retlist = g_slist_insert_sorted (retlist, def, (GCompareFunc) def_cmp_func);
+		}
+	}
+
+	/* handle parents */
+	GSList *list;
+	for (list = kl->parents; list; list = list->next)
+		retlist = handle_ldap_class (cdata, (GdaLdapClass*) list->data, retlist, hash);
+
+	return retlist;
+}
+
+GSList *
+gdaprov_ldap_get_attributes_list (GdaLdapConnection *cnc, GdaLdapAttribute *object_class_attr)
+{
+	LdapConnectionData *cdata;
+	g_return_val_if_fail (GDA_IS_LDAP_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (object_class_attr, NULL);
+
+	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
+        if (!cdata)
+                return NULL;
+
+	guint i;
+	const gchar *tmp;
+	GHashTable *hash;
+	GSList *retlist = NULL;
+
+	hash = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i = 0; i < object_class_attr->nb_values; i++) {
+		if (G_VALUE_TYPE (object_class_attr->values [i]) != G_TYPE_STRING) {
+			g_warning (_("Unexpected data type '%s' for objectClass attribute!"),
+				   gda_g_type_to_string (G_VALUE_TYPE (object_class_attr->values [i])));
+			continue;
+		}
+		tmp = g_value_get_string (object_class_attr->values [i]);
+
+		GdaLdapClass *kl;
+		kl = gdaprov_ldap_get_class_info (cnc, tmp);
+		if (!kl) {
+			g_warning (_("Can't get information about '%s' class"), tmp);
+			continue;
+		}
+		retlist = handle_ldap_class (cdata, kl, retlist, hash);
+	}
+	g_hash_table_destroy (hash);
+
+	return retlist;
+}
