@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -16,10 +16,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-#include <glib/gi18n-lib.h>
 #include <libgda-ui/libgda-ui.h>
 #include <libgda-ui/gdaui-plugin.h>
 #include <gtk/gtk.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 typedef enum {
 	TESTED_BASIC,
@@ -42,7 +44,7 @@ typedef struct {
 	GType      *tested_gtypes;
 	gint        nb_tested_gtypes;
 	GHashTable *models_hash; /* key = GType, value = a GdaDataModel to test */
-	GHashTable *tests_hash; /* key = a GtkVBox, value = a TestWidgetData */
+	GHashTable *tests_hash; /* key = a GtkBox, value = a TestWidgetData */
 } TestConfig;
 
 extern GHashTable *gdaui_plugins_hash;
@@ -122,8 +124,10 @@ main (int argc, char **argv)
 	GOptionContext *context;
 	GError *error = NULL;
 
+#ifdef HAVE_LOCALE_H
 	/* Initialize i18n support */
-	gtk_set_locale ();
+	setlocale (LC_ALL,"");
+#endif
 
 	/* command line parsing */
         context = g_option_context_new ("Gdaui entry widgets and cell renderers testing");
@@ -141,7 +145,7 @@ main (int argc, char **argv)
 	/* init main conf */
 	GType tested_gtypes [] = {G_TYPE_INT64, G_TYPE_UINT64, GDA_TYPE_BINARY, G_TYPE_BOOLEAN, GDA_TYPE_BLOB,
 				  G_TYPE_DATE, G_TYPE_DOUBLE,
-				  GDA_TYPE_GEOMETRIC_POINT, G_TYPE_OBJECT, G_TYPE_INT, GDA_TYPE_LIST, 
+				  GDA_TYPE_GEOMETRIC_POINT, G_TYPE_OBJECT, G_TYPE_INT, 
 				  GDA_TYPE_NUMERIC, G_TYPE_FLOAT, GDA_TYPE_SHORT, GDA_TYPE_USHORT, G_TYPE_STRING, 
 				  GDA_TYPE_TIME, GDA_TYPE_TIMESTAMP, G_TYPE_CHAR, G_TYPE_UCHAR, G_TYPE_UINT};
 	mainconf.test_type = TESTED_BASIC;
@@ -165,7 +169,7 @@ main (int argc, char **argv)
 	g_signal_connect (G_OBJECT (mainwin), "destroy",
 			  G_CALLBACK (destroy), NULL);
 
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add (GTK_CONTAINER (mainwin), vbox);
 	gtk_widget_show (vbox);
 
@@ -181,7 +185,7 @@ main (int argc, char **argv)
 	/*
 	 * Default handlers
 	 */
-	vbox = gtk_vbox_new (FALSE, 10);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 10);
 	gtk_notebook_append_page (GTK_NOTEBOOK (top_nb), vbox, NULL);
 	gtk_widget_show (vbox);
 
@@ -205,7 +209,7 @@ main (int argc, char **argv)
 	/*
 	 * Plugins page 
 	 */
-	vbox = gtk_vbox_new (FALSE, 10);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 10);
 	gtk_notebook_append_page (GTK_NOTEBOOK (top_nb), vbox, NULL);
 	gtk_widget_show (vbox);
 
@@ -250,7 +254,7 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	menubar1 = gtk_menu_bar_new ();
 	gtk_widget_show (menubar1);
 
-	menuitem1 = gtk_menu_item_new_with_mnemonic (_("_File"));
+	menuitem1 = gtk_menu_item_new_with_mnemonic ("_File");
 	gtk_widget_show (menuitem1);
 	gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);
 	
@@ -263,28 +267,28 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (destroy), NULL);
 
-	menuitem2 = gtk_menu_item_new_with_mnemonic (_("_Tested Widgets"));
+	menuitem2 = gtk_menu_item_new_with_mnemonic ("_Tested Widgets");
 	gtk_widget_show (menuitem2);
 	gtk_container_add (GTK_CONTAINER (menubar1), menuitem2);
 	
 	menuitem2_menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menuitem2_menu);
 	
-	mitem = gtk_menu_item_new_with_mnemonic (_("Default individual data entry widgets"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Default individual data entry widgets");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_BASIC));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_default_handlers_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic (_("Default data entry widgets in a form"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Default data entry widgets in a form");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_FORM));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_default_handlers_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic (_("Default data cell renderers in a grid"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Default data cell renderers in a grid");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_GRID));
@@ -294,21 +298,21 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	mitem = gtk_separator_menu_item_new ();
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 
-	mitem = gtk_menu_item_new_with_mnemonic (_("Plugins individual data entry widgets"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data entry widgets alone");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_BASIC));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_plugins_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic (_("Plugins data entry widgets in a form"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data entry widgets in a form");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_FORM));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_plugins_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic (_("Plugins data cell renderers in a grid"));
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data cell renderers");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_GRID));
@@ -506,7 +510,7 @@ create_plugin_nb (GtkWidget *table, GdauiPlugin *plugin)
 			if (dh) {
 				TestWidgetData *tdata;
 
-				wid = gtk_vbox_new (FALSE, 0);
+				wid = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 				tdata = g_new0 (TestWidgetData, 1);
 				tdata->vbox = wid;
 				tdata->type = type;
@@ -531,7 +535,7 @@ create_plugin_nb (GtkWidget *table, GdauiPlugin *plugin)
 			if (dh) {
 				TestWidgetData *tdata;
 
-				wid = gtk_vbox_new (FALSE, 0);
+				wid = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 				tdata = g_new0 (TestWidgetData, 1);
 				tdata->vbox = wid;
 				tdata->type = type;
@@ -561,7 +565,7 @@ static GtkWidget *build_form_test_for_gtype (GdaDataHandler *dh, GType type, con
 static GtkWidget *build_grid_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_name);
 
 static void
-plugin_nb_page_changed_cb (GtkNotebook *nb, G_GNUC_UNUSED GtkWidget *page, gint pageno, 
+plugin_nb_page_changed_cb (GtkNotebook *nb, G_GNUC_UNUSED GtkWidget *page, gint pageno,
 			   G_GNUC_UNUSED GtkWidget *table)
 {
 	GtkWidget *vbox;
@@ -676,7 +680,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 	GString *string;
 	gboolean expand;
 	
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
 	table = gtk_table_new (7, 3, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
@@ -690,7 +694,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 					gda_data_handler_get_descr (dh));
 	else
 		g_string_append_printf (string, "<b>Data handler description:</b> %s\n",
-					_("No GdaDataHandler available for this type"));
+					"No GdaDataHandler available for this type");
 	label = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_label_set_markup (GTK_LABEL (label), string->str);
@@ -710,18 +714,18 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 	gtk_widget_show (wid);
 
 	/* Other widgets */
-	label = gtk_label_new (_("Current flags: "));
+	label = gtk_label_new ("Current flags: ");
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
 	gtk_widget_show (label);
 
-	label = gtk_label_new (_("--"));
+	label = gtk_label_new ("--");
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 2, 3, 0, 0, 0, 0);
 	g_object_set_data (G_OBJECT (wid), "flags", label);
 	gtk_widget_show (label);
 
-	label = gtk_label_new (_("Current value: "));
+	label = gtk_label_new ("Current value: ");
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
 	gtk_widget_show (label);
@@ -736,10 +740,10 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 
 	entry_contents_modified (wid, GTK_LABEL (label));
 
-	bbox = gtk_hbutton_box_new ();
+	bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
 	gtk_table_attach (GTK_TABLE (table), bbox, 0, 3, 4, 5, 0, 0, 0, 0);
 		
-	button = gtk_toggle_button_new_with_label (_("NULL ok"));
+	button = gtk_toggle_button_new_with_label ("NULL ok");
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (null_toggled_cb), wid);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
@@ -747,7 +751,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (wid)) & 
 				      GDA_VALUE_ATTR_CAN_BE_NULL);
 
-	button = gtk_toggle_button_new_with_label (_("DEFAULT ok"));
+	button = gtk_toggle_button_new_with_label ("DEFAULT ok");
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (default_toggled_cb), wid);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
@@ -755,7 +759,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (wid)) & 
 				      GDA_VALUE_ATTR_CAN_BE_DEFAULT);
 
-	button = gtk_toggle_button_new_with_label (_("Actions?"));
+	button = gtk_toggle_button_new_with_label ("Actions?");
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (actions_toggled_cb), wid);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
@@ -763,7 +767,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (wid)) & 
 				      GDA_VALUE_ATTR_ACTIONS_SHOWN);
 
-	button = gtk_toggle_button_new_with_label (_("Editable?"));
+	button = gtk_toggle_button_new_with_label ("Editable?");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (editable_toggled_cb), wid);
@@ -780,7 +784,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 	else
 		gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 5, 6, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_widget_show (entry);
-	button = gtk_button_new_with_label (_("Set as original"));
+	button = gtk_button_new_with_label ("Set as original");
 	g_object_set_data (G_OBJECT (button), "entry", entry);
 	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 5, 6, 0, 0, 0, 0);
 	gtk_widget_show (button);
@@ -796,7 +800,7 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 	else
 		gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 6, 7, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	gtk_widget_show (entry);
-	button = gtk_button_new_with_label (_("Set as default"));
+	button = gtk_button_new_with_label ("Set as default");
 	g_object_set_data (G_OBJECT (button), "entry", entry);
 	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 6, 7, 0, 0, 0, 0);
 	gtk_widget_show (button);
