@@ -130,7 +130,7 @@ data_model_errors_init (DataModelErrors *model,
 		value = gda_row_get_value (row, 0);
 		if (arow->col0) {
 			if (*arow->col0 == '-')
-				G_VALUE_TYPE (value) = G_MAXINT;
+				G_VALUE_TYPE (value) = G_TYPE_INVALID;
 			else {
 				gda_value_reset_with_type (value, G_TYPE_STRING);
 				g_value_set_string (value, arow->col0);
@@ -140,7 +140,7 @@ data_model_errors_init (DataModelErrors *model,
 		value = gda_row_get_value (row, 1);
 		if (arow->col1) {
 			if (*arow->col1 == '-')
-				G_VALUE_TYPE (value) = G_MAXINT;
+				G_VALUE_TYPE (value) = G_TYPE_INVALID;
 			else {
 				gda_value_reset_with_type (value, G_TYPE_STRING);
 				g_value_set_string (value, arow->col1);
@@ -150,7 +150,7 @@ data_model_errors_init (DataModelErrors *model,
 		value = gda_row_get_value (row, 2);
 		if (arow->col2) {
 			if (*arow->col2 == '-')
-				G_VALUE_TYPE (value) = G_MAXINT;
+				G_VALUE_TYPE (value) = G_TYPE_INVALID;
 			else {
 				gda_value_reset_with_type (value, G_TYPE_STRING);
 				g_value_set_string (value, arow->col2);
@@ -160,14 +160,13 @@ data_model_errors_init (DataModelErrors *model,
 		value = gda_row_get_value (row, 3);
 		if (arow->col3) {
 			if (*arow->col3 == '-')
-				G_VALUE_TYPE (value) = G_MAXINT;
+				G_VALUE_TYPE (value) = G_TYPE_INVALID;
 			else {
 				gda_value_reset_with_type (value, G_TYPE_STRING);
 				g_value_set_string (value, arow->col3);
 			}
 		}
 
-
 		g_ptr_array_add (model->priv->rows, row);
 	}
 }
@@ -198,7 +197,7 @@ data_model_errors_dispose (GObject * object)
                 }
 
 		/* DONT: g_ptr_array_foreach (model->priv->rows, (GFunc) g_object_unref, NULL);
-		 * because we use the convention that G_VALUE_TYPE() == G_MAXINT for errors */
+		 * because we use the convention that G_VALUE_TYPE() == G_TYPE_INVALID for errors */
 		g_ptr_array_free (model->priv->rows, TRUE);
 		g_free (model->priv);
 		model->priv = NULL;
@@ -345,7 +344,7 @@ data_model_errors_get_value_at (GdaDataModel *model, gint col, gint row, GError
 	drow =  g_ptr_array_index (imodel->priv->rows, row);
 	if (drow) {
 		GValue *val = gda_row_get_value (drow, col);
-		if (G_VALUE_TYPE (val) == G_MAXINT) {
+		if (G_VALUE_TYPE (val) == G_TYPE_INVALID) {
 			/* simulates an error */
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
 				     "%s", "Simulated error");
@@ -376,7 +375,7 @@ data_model_errors_get_attributes_at (GdaDataModel *model, gint col, G_GNUC_UNUSE
 		return 0;
 	}
 
-	flags = GDA_VALUE_ATTR_NO_MODIF;
+	flags = 0;
 	return flags;
 }
 
@@ -402,12 +401,16 @@ data_model_errors_set_value_at (GdaDataModel *model, gint col, gint row, const G
 	drow =  g_ptr_array_index (imodel->priv->rows, row);
 	if (drow) {
 		GValue *dvalue;
-		dvalue = gda_row_get_value (drow, row);
+		dvalue = gda_row_get_value (drow, col);
+		gda_value_reset_with_type (dvalue, G_VALUE_TYPE (value));
 		g_value_copy (value, dvalue);
+		gda_data_model_row_updated (model, row);
 	}
-	else
+	else {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
 			      "%s", "Row not found");
+		retval = FALSE;
+	}
 
 	return retval;
 }
